# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import copy
from unittest import TestCase, mock

from delfin import context
from delfin import exception

import sys
sys.modules['delfin.cryptor'] = mock.Mock()

from delfin.drivers.api import API
from delfin.drivers.fake_storage import FakeStorageDriver


class Request:
    def __init__(self):
        self.environ = {'delfin.context': context.RequestContext()}
        pass


ACCESS_INFO = {
    "storage_id": "12345",
    "vendor": "fake_storage",
    "model": "fake_driver",
    "rest": {
        "host": "10.0.0.1",
        "port": "8443",
        "username": "user",
        "password": "pass"
    },
    "extra_attributes": {
        "array_id": "00112233"
    }
}

STORAGE = {
    'name': 'fake_driver',
    'description': 'it is a fake driver.',
    'vendor': 'fake_vendor',
    'model': 'fake_model',
    'status': 'normal',
    'serial_number': '2102453JPN12KA000011',
    'firmware_version': '1.0.0',
    'location': 'HK',
    'total_capacity': 1024 * 1024,
    'used_capacity': 3126,
    'free_capacity': 1045449,
}


class TestDriverAPI(TestCase):

    def test_init(self):
        api = API()
        self.assertIsNotNone(api.driver_manager)

    @mock.patch('delfin.db.storage_create')
    @mock.patch('delfin.db.access_info_create')
    @mock.patch('delfin.db.storage_get_all')
    def test_discover_storage(self, mock_storage, mock_access_info,
                              mock_storage_create):

        # Case: Positive scenario for fake driver discovery
        storage = copy.deepcopy(STORAGE)
        storage['id'] = '12345'
        mock_storage.return_value = None
        mock_access_info.return_value = ACCESS_INFO
        mock_storage_create.return_value = storage
        api = API()
        api.discover_storage(context, ACCESS_INFO)
        mock_storage.assert_called()
        mock_access_info.assert_called_with(context, ACCESS_INFO)
        mock_storage_create.assert_called()

        # Case: Register already existing storage
        with self.assertRaises(exception.StorageAlreadyExists) as exc:
            mock_storage.return_value = storage
            api.discover_storage(context, ACCESS_INFO)
        self.assertIn('Storage already exists', str(exc.exception))
        mock_storage.return_value = None

        # Case: Storage without serial number
        wrong_storage = copy.deepcopy(STORAGE)
        wrong_storage.pop('serial_number')
        wrong_storage['id'] = '12345'
        m = mock.Mock()
        with mock.patch.object(FakeStorageDriver, 'get_storage') as m:
            with self.assertRaises(exception.InvalidResults) as exc:
                m.return_value = wrong_storage
                api.discover_storage(context, ACCESS_INFO)
            self.assertIn('Serial number should be provided by storage',
                          str(exc.exception))

            # Case: No Storage found
            with self.assertRaises(exception.StorageBackendNotFound) as exc:
                m.return_value = None
                api.discover_storage(context, ACCESS_INFO)
            self.assertIn('Storage backend could not be found',
                          str(exc.exception))

        # Case: Test access info without 'storage_id' for driver
        test_access_info = copy.deepcopy(ACCESS_INFO)
        test_access_info.pop('storage_id')

        s = api.discover_storage(context, ACCESS_INFO)
        self.assertDictEqual(s, storage)

        # Case: Wrong access info (model) for driver
        wrong_access_info = copy.deepcopy(ACCESS_INFO)
        wrong_access_info['model'] = 'wrong_model'
        with self.assertRaises(exception.StorageDriverNotFound) as exc:
            api.discover_storage(context, wrong_access_info)

        msg = "Storage driver 'fake_storage wrong_model'could not be found"
        self.assertIn(msg, str(exc.exception))

    @mock.patch.object(FakeStorageDriver, 'get_storage')
    @mock.patch('delfin.db.storage_update')
    @mock.patch('delfin.db.access_info_update')
    @mock.patch('delfin.db.storage_get')
    def test_update_access_info(self, mock_storage_get,
                                mock_access_info_update,
                                mock_storage_update,
                                mock_storage):
        storage = copy.deepcopy(STORAGE)
        access_info_new = copy.deepcopy(ACCESS_INFO)
        access_info_new['rest']['username'] = 'new_user'

        mock_storage_get.return_value = storage
        mock_access_info_update.return_value = access_info_new
        mock_storage_update.return_value = None
        mock_storage.return_value = storage
        api = API()
        updated = api.update_access_info(context, access_info_new)

        storage_id = '12345'
        mock_storage_get.assert_called_with(
            context, storage_id)

        mock_access_info_update.assert_called_with(
            context, storage_id, access_info_new)

        mock_storage_update.assert_called_with(
            context, storage_id, storage)

        access_info_new['rest']['password'] = mock.Mock()
        self.assertDictEqual(access_info_new, updated)

        # Case: Wrong storage serial number
        wrong_storage = copy.deepcopy(STORAGE)
        wrong_storage['serial_number'] = '00000'
        mock_storage.return_value = wrong_storage
        with self.assertRaises(exception.StorageSerialNumberMismatch) as exc:
            api.update_access_info(context, access_info_new)

        msg = "Serial number 00000 does not match " \
              "the existing storage serial number"
        self.assertIn(msg, str(exc.exception))

        # Case: No storage serial number
        wrong_storage.pop('serial_number')
        mock_storage.return_value = wrong_storage
        with self.assertRaises(exception.InvalidResults) as exc:
            api.update_access_info(context, access_info_new)

        msg = "Serial number should be provided by storage"
        self.assertIn(msg, str(exc.exception))

        # Case: No storage
        mock_storage.return_value = None
        with self.assertRaises(exception.StorageBackendNotFound) as exc:
            api.update_access_info(context, access_info_new)

        msg = "Storage backend could not be found"
        self.assertIn(msg, str(exc.exception))

    @mock.patch('delfin.db.storage_create')
    @mock.patch('delfin.db.access_info_create')
    @mock.patch('delfin.db.storage_get_all')
    def test_remove_storage(self, mock_storage, mock_access_info,
                            mock_storage_create):
        storage = copy.deepcopy(STORAGE)
        storage['id'] = '12345'
        mock_storage.return_value = None
        mock_access_info.return_value = ACCESS_INFO
        mock_storage_create.return_value = storage
        api = API()
        api.discover_storage(context, ACCESS_INFO)

        storage_id = '12345'
        driver = api.driver_manager.driver_factory.get(storage_id, None)
        self.assertIsNotNone(driver)

        api.remove_storage(context, storage_id)

        driver = api.driver_manager.driver_factory.get(storage_id, None)
        self.assertIsNone(driver)

    @mock.patch.object(FakeStorageDriver, 'get_storage')
    @mock.patch('delfin.db.storage_create')
    @mock.patch('delfin.db.access_info_create')
    @mock.patch('delfin.db.storage_get_all')
    def test_get_storage(self, mock_storage, mock_access_info,
                         mock_storage_create, mock_fake):
        storage = copy.deepcopy(STORAGE)
        storage['id'] = '12345'
        mock_storage.return_value = None
        mock_access_info.return_value = ACCESS_INFO
        mock_storage_create.return_value = storage
        mock_fake.return_value = storage
        api = API()
        api.discover_storage(context, ACCESS_INFO)

        storage_id = '12345'
        driver = api.driver_manager.driver_factory.get(storage_id, None)
        self.assertIsNotNone(driver)

        api.get_storage(context, storage_id)
        mock_fake.assert_called()

    @mock.patch.object(FakeStorageDriver, 'list_storage_pools')
    @mock.patch('delfin.db.storage_create')
    @mock.patch('delfin.db.access_info_create')
    @mock.patch('delfin.db.storage_get_all')
    def test_list_storage_pools(self, mock_storage, mock_access_info,
                                mock_storage_create, mock_fake):
        storage = copy.deepcopy(STORAGE)
        storage['id'] = '12345'
        mock_storage.return_value = None
        mock_access_info.return_value = ACCESS_INFO
        mock_storage_create.return_value = storage
        mock_fake.return_value = []
        api = API()
        api.discover_storage(context, ACCESS_INFO)

        storage_id = '12345'
        driver = api.driver_manager.driver_factory.get(storage_id, None)
        self.assertIsNotNone(driver)

        api.list_storage_pools(context, storage_id)
        mock_fake.assert_called_once()

    @mock.patch.object(FakeStorageDriver, 'list_volumes')
    @mock.patch('delfin.db.storage_create')
    @mock.patch('delfin.db.access_info_create')
    @mock.patch('delfin.db.storage_get_all')
    def test_list_volumes(self, mock_storage, mock_access_info,
                          mock_storage_create, mock_fake):
        storage = copy.deepcopy(STORAGE)
        storage['id'] = '12345'
        mock_storage.return_value = None
        mock_access_info.return_value = ACCESS_INFO
        mock_storage_create.return_value = storage
        mock_fake.return_value = []
        api = API()
        api.discover_storage(context, ACCESS_INFO)

        storage_id = '12345'
        driver = api.driver_manager.driver_factory.get(storage_id, None)
        self.assertIsNotNone(driver)

        api.list_volumes(context, storage_id)
        mock_fake.assert_called_once()

    @mock.patch.object(FakeStorageDriver, 'parse_alert')
    @mock.patch('delfin.db.storage_create')
    @mock.patch('delfin.db.access_info_create')
    @mock.patch('delfin.db.storage_get_all')
    def test_parse_alert(self, mock_storage, mock_access_info,
                         mock_storage_create, mock_fake):
        storage = copy.deepcopy(STORAGE)
        storage['id'] = '12345'
        mock_storage.return_value = None
        mock_access_info.return_value = ACCESS_INFO
        mock_storage_create.return_value = storage
        mock_fake.return_value = []
        api = API()
        api.discover_storage(context, ACCESS_INFO)

        storage_id = '12345'
        driver = api.driver_manager.driver_factory.get(storage_id, None)
        self.assertIsNotNone(driver)

        api.parse_alert(context, storage_id, 'alert')
        mock_fake.assert_called_once()



# Copyright 2020 The SODA Authors.
# Copyright (C) 2017 NTT DATA
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Internal implementation of request Body validating middleware.

"""

import re

import jsonschema
from jsonschema import exceptions as jsonschema_exc
from oslo_utils import timeutils
from oslo_utils import uuidutils
import six

from delfin import exception
from delfin.i18n import _
from delfin import utils


def _soft_validate_additional_properties(
        validator, additional_properties_value, param_value, schema):
    """Validator function.

    If there are not any properties on the param_value that are not specified
    in the schema, this will return without any effect. If there are any such
    extra properties, they will be handled as follows:

    - if the validator passed to the method is not of type "object", this
      method will return without any effect.
    - if the 'additional_properties_value' parameter is True, this method will
      return without any effect.
    - if the schema has an additionalProperties value of True, the extra
      properties on the param_value will not be touched.
    - if the schema has an additionalProperties value of False and there
      aren't patternProperties specified, the extra properties will be stripped
      from the param_value.
    - if the schema has an additionalProperties value of False and there
      are patternProperties specified, the extra properties will not be
      touched and raise validation error if pattern doesn't match.
    """
    if (not validator.is_type(param_value, "object") or
            additional_properties_value):
        return

    properties = schema.get("properties", {})
    patterns = "|".join(schema.get("patternProperties", {}))
    extra_properties = set()
    for prop in param_value:
        if prop not in properties:
            if patterns:
                if not re.search(patterns, prop):
                    extra_properties.add(prop)
            else:
                extra_properties.add(prop)

    if not extra_properties:
        return

    if patterns:
        error = "Additional properties are not allowed (%s %s unexpected)"
        if len(extra_properties) == 1:
            verb = "was"
        else:
            verb = "were"
        yield jsonschema_exc.ValidationError(
            error % (", ".join(repr(extra) for extra in extra_properties),
                     verb))
    else:
        for prop in extra_properties:
            del param_value[prop]


def _validate_string_length(value, entity_name, mandatory=False,
                            min_length=0, max_length=None,
                            remove_whitespaces=False):
    """Check the length of specified string.

    :param value: the value of the string
    :param entity_name: the name of the string
    :mandatory: string is mandatory or not
    :param min_length: the min_length of the string
    :param max_length: the max_length of the string
    :param remove_whitespaces: True if trimming whitespaces is needed
                                   else False
    """
    if not mandatory and not value:
        return True

    if mandatory and not value:
        msg = _("The '%s' can not be None.") % entity_name
        raise exception.InvalidInput(msg)

    if remove_whitespaces:
        value = value.strip()

    utils.check_string_length(value, entity_name,
                              min_length=min_length,
                              max_length=max_length)


@jsonschema.FormatChecker.cls_checks('date-time')
def _validate_datetime_format(param_value):
    try:
        timeutils.parse_isotime(param_value)
    except ValueError:
        return False
    else:
        return True


@jsonschema.FormatChecker.cls_checks('name', exception.InvalidName)
def _validate_name(param_value):
    if not param_value:
        msg = _("The 'name' can not be None.")
        raise exception.InvalidName(msg)
    elif len(param_value.strip()) == 0:
        msg = _("The 'name' can not be empty.")
        raise exception.InvalidName(msg)
    return True


@jsonschema.FormatChecker.cls_checks('name_skip_leading_trailing_spaces',
                                     exception.InvalidName)
def _validate_name_skip_leading_trailing_spaces(param_value):
    if not param_value:
        msg = _("The 'name' can not be None.")
        raise exception.InvalidName(msg)
    param_value = param_value.strip()
    if len(param_value) == 0:
        msg = _("The 'name' can not be empty.")
        raise exception.InvalidName(msg)
    elif len(param_value) > 255:
        msg = _("The 'name' can not be greater than 255 characters.")
        raise exception.InvalidName(msg)
    return True


@jsonschema.FormatChecker.cls_checks('uuid')
def _validate_uuid_format(instance):
    return uuidutils.is_uuid_like(instance)


class FormatChecker(jsonschema.FormatChecker):
    """A FormatChecker can output the message from cause exception

    We need understandable validation errors messages for users. When a
    custom checker has an exception, the FormatChecker will output a
    readable message provided by the checker.
    """

    def check(self, param_value, format):
        """Check whether the param_value conforms to the given format.

        :argument param_value: the param_value to check
        :type: any primitive type (str, number, bool)
        :argument str format: the format that param_value should conform to
        :raises: :exc:`FormatError` if param_value does not conform to format
        """

        if format not in self.checkers:
            return

        # For safety reasons custom checkers can be registered with
        # allowed exception types. Anything else will fall into the
        # default formatter.
        func, raises = self.checkers[format]
        result, cause = None, None

        try:
            result = func(param_value)
        except raises as e:
            cause = e
        if not result:
            msg = "%r is not a %r" % (param_value, format)
            raise jsonschema_exc.FormatError(msg, cause=cause)


class _SchemaValidator(object):
    """A validator class

    This class is changed from Draft4Validator to validate minimum/maximum
    value of a string number(e.g. '10'). This changes can be removed when
    we tighten up the API definition and the XML conversion.
    Also FormatCheckers are added for checking data formats which would be
    passed through cinder api commonly.

    """
    validator = None
    validator_org = jsonschema.Draft4Validator

    def __init__(self, schema, relax_additional_properties=False):
        validators = {
            'minimum': self._validate_minimum,
            'maximum': self._validate_maximum,
        }
        if relax_additional_properties:
            validators[
                'additionalProperties'] = _soft_validate_additional_properties

        validator_cls = jsonschema.validators.extend(self.validator_org,
                                                     validators)
        format_checker = FormatChecker()
        self.validator = validator_cls(schema, format_checker=format_checker)

    def validate(self, *args, **kwargs):
        try:
            self.validator.validate(*args, **kwargs)
        except jsonschema.ValidationError as ex:
            if isinstance(ex.cause, exception.InvalidName):
                raise ex.cause
            elif len(ex.path) > 0:
                detail = _("Invalid input for field/attribute %(path)s."
                           " %(message)s") % {'path': ex.path.pop(),
                                              'message': ex.message}
            else:
                detail = ex.message
            raise exception.InvalidInput(detail)
        except TypeError as ex:
            # NOTE: If passing non string value to patternProperties parameter,
            #       TypeError happens. Here is for catching the TypeError.
            detail = six.text_type(ex)
            raise exception.InvalidInput(detail)

    def _number_from_str(self, param_value):
        try:
            value = int(param_value)
        except (ValueError, TypeError):
            try:
                value = float(param_value)
            except (ValueError, TypeError):
                return None
        return value

    def _validate_minimum(self, validator, minimum, param_value, schema):
        param_value = self._number_from_str(param_value)
        if param_value is None:
            return
        return self.validator_org.VALIDATORS['minimum'](validator, minimum,
                                                        param_value, schema)

    def _validate_maximum(self, validator, maximum, param_value, schema):
        param_value = self._number_from_str(param_value)
        if param_value is None:
            return
        return self.validator_org.VALIDATORS['maximum'](validator, maximum,
                                                        param_value, schema)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from datetime import datetime

from oslo_log import log

from delfin import exception
from delfin.common import constants
from delfin.drivers.huawei.oceanstor import oid_mapper
from delfin.i18n import _

LOG = log.getLogger(__name__)


class AlertHandler(object):
    """Alert handling functions for huawei oceanstor driver"""
    def __init__(self):
        self.oid_mapper = oid_mapper.OidMapper()

    TIME_PATTERN = "%Y-%m-%d,%H:%M:%S.%f"

    # Translation of trap severity to alert model severity
    # Values are: criticalAlarm=1, majorAlarm=2, minorAlarm=3, warningAlarm=4
    SEVERITY_MAP = {"1": constants.Severity.CRITICAL,
                    "2": constants.Severity.MAJOR,
                    "3": constants.Severity.MINOR,
                    "4": constants.Severity.WARNING}

    # Translation of trap alert category to alert model category
    # Values are: faultAlarm=1, resumeAlarm=2, eventAlarm=3
    CATEGORY_MAP = {"1": constants.Category.FAULT,
                    "2": constants.Category.RECOVERY,
                    "3": constants.Category.EVENT}

    # Translation of trap alert category to alert type
    # Values are: communicationQuality=1, equipmentFault=2, processError=3
    #             serviceQuality=4, environmentFault=5, performanceLimit=6
    TYPE_MAP = {
        "1": constants.EventType.COMMUNICATIONS_ALARM,
        "2": constants.EventType.EQUIPMENT_ALARM,
        "3": constants.EventType.PROCESSING_ERROR_ALARM,
        "4": constants.EventType.QUALITY_OF_SERVICE_ALARM,
        "5": constants.EventType.ENVIRONMENTAL_ALARM,
        "6": constants.EventType.QUALITY_OF_SERVICE_ALARM}

    # Translation of severity of queried alerts to alert model severity
    QUERY_ALERTS_SEVERITY_MAP = {2: constants.Severity.INFORMATIONAL,
                                 3: constants.Severity.WARNING,
                                 5: constants.Severity.MAJOR,
                                 6: constants.Severity.CRITICAL}

    # Translation of alert category of queried alerts to alert model category
    QUERY_ALERTS_CATEGORY_MAP = {0: constants.Category.EVENT,
                                 1: constants.Category.FAULT,
                                 2: constants.Category.RECOVERY}

    # Attributes expected in alert info to proceed with model filling
    _mandatory_alert_attributes = ('hwIsmReportingAlarmAlarmID',
                                   'hwIsmReportingAlarmFaultTitle',
                                   'hwIsmReportingAlarmFaultLevel',
                                   'hwIsmReportingAlarmNodeCode',
                                   'hwIsmReportingAlarmFaultType',
                                   'hwIsmReportingAlarmAdditionInfo',
                                   'hwIsmReportingAlarmSerialNo',
                                   'hwIsmReportingAlarmFaultCategory',
                                   'hwIsmReportingAlarmRestoreAdvice',
                                   'hwIsmReportingAlarmFaultTime'
                                   )

    def parse_alert(self, context, alert):
        """Parse alert data and fill the alert model."""
        # Check for mandatory alert attributes
        alert = self.oid_mapper.map_oids(alert)
        LOG.info("Get alert from storage: %s", alert)

        for attr in self._mandatory_alert_attributes:
            if not alert.get(attr):
                msg = "Mandatory information %s missing in alert message. " \
                      % attr
                raise exception.InvalidInput(msg)

        try:
            alert_model = dict()
            # These information are sourced from device registration info
            alert_model['alert_id'] = alert['hwIsmReportingAlarmAlarmID']
            alert_model['alert_name'] = alert['hwIsmReportingAlarmFaultTitle']
            alert_model['severity'] = self.SEVERITY_MAP.get(
                alert['hwIsmReportingAlarmFaultLevel'],
                constants.Severity.NOT_SPECIFIED)
            alert_model['category'] = self.CATEGORY_MAP.get(
                alert['hwIsmReportingAlarmFaultCategory'],
                constants.Category.NOT_SPECIFIED)
            alert_model['type'] = self.TYPE_MAP.get(
                alert['hwIsmReportingAlarmFaultType'],
                constants.EventType.NOT_SPECIFIED)
            alert_model['sequence_number'] \
                = alert['hwIsmReportingAlarmSerialNo']
            occur_time = datetime.strptime(
                alert['hwIsmReportingAlarmFaultTime'],
                self.TIME_PATTERN)
            alert_model['occur_time'] = int(occur_time.timestamp() * 1000)

            description = alert['hwIsmReportingAlarmAdditionInfo']
            if self._is_hex(description):
                description = bytes.fromhex(description[2:]).decode('ascii')
            alert_model['description'] = description

            recovery_advice = alert['hwIsmReportingAlarmRestoreAdvice']
            if self._is_hex(recovery_advice):
                recovery_advice = bytes.fromhex(
                    recovery_advice[2:]).decode('ascii')

            alert_model['recovery_advice'] = recovery_advice

            alert_model['resource_type'] = constants.DEFAULT_RESOURCE_TYPE
            alert_model['location'] = 'Node code=' \
                                      + alert['hwIsmReportingAlarmNodeCode']

            if alert.get('hwIsmReportingAlarmLocationInfo'):
                alert_model['location'] \
                    = alert_model['location'] + ',' + alert[
                    'hwIsmReportingAlarmLocationInfo']

            return alert_model
        except Exception as e:
            LOG.error(e)
            msg = (_("Failed to build alert model as some attributes missing "
                     "in alert message."))
            raise exception.InvalidResults(msg)

    def parse_queried_alerts(self, alert_list):
        """Parses list alert data and fill the alert model."""
        # List contains all the current alarms of given storage id
        alert_model_list = []
        for alert in alert_list:
            try:
                alert_model = dict()
                alert_model['alert_id'] = alert['eventID']
                alert_model['alert_name'] = alert['name']
                alert_model['severity'] = self.QUERY_ALERTS_SEVERITY_MAP.get(
                    alert['level'], constants.Severity.NOT_SPECIFIED)
                alert_model['category'] = self.QUERY_ALERTS_CATEGORY_MAP.get(
                    alert['eventType'], constants.Category.NOT_SPECIFIED)
                alert_model['type'] = constants.EventType.NOT_SPECIFIED
                alert_model['sequence_number'] = alert['sequence']
                occur_time = alert['startTime']
                alert_model['occur_time'] = int(occur_time * 1000)
                alert_model['description'] = alert['description']

                alert_model['recovery_advice'] = alert['suggestion']

                alert_model['resource_type'] = constants.DEFAULT_RESOURCE_TYPE
                alert_model['location'] = alert['location']

                alert_model_list.append(alert_model)
            except Exception as e:
                LOG.error(e)
                msg = (_("Failed to build alert model as some attributes"
                         " missing in queried alerts."))
                raise exception.InvalidResults(msg)
        return alert_model_list

    def add_trap_config(self, context, storage_id, trap_config):
        """Config the trap receiver in storage system."""
        # Currently not implemented
        pass

    def remove_trap_config(self, context, storage_id, trap_config):
        """Remove trap receiver configuration from storage system."""
        # Currently not implemented
        pass

    def clear_alert(self, context, storage_id, alert):
        # Currently not implemented
        """Clear alert from storage system."""
        pass

    def _is_hex(self, value):
        try:
            int(value, 16)
        except ValueError:
            return False
        return True



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import webob.dec

from delfin import context
from delfin.wsgi import common as wsgi


class ContextWrapper(wsgi.Middleware):
    """Add 'delfin.context' to req.environ"""

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        req.environ['delfin.context'] = context.RequestContext()
        return self.application



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from oslo_utils import importutils
from pysnmp.carrier.asyncore.dgram import udp
from pysnmp.entity import engine, config

from delfin import exception
from delfin import test
from delfin.tests.unit.alert_manager import fakes


class TrapReceiverTestCase(test.TestCase):
    TRAP_RECEIVER_CLASS = 'delfin.alert_manager.trap_receiver' \
                          '.TrapReceiver'
    DEF_TRAP_RECV_ADDR = '127.0.0.1'
    DEF_TRAP_RECV_PORT = '162'

    def setUp(self):
        super(TrapReceiverTestCase, self).setUp()
        self.alert_rpc_api = mock.Mock()
        trap_receiver_class = importutils.import_class(
            self.TRAP_RECEIVER_CLASS)
        self.trap_receiver = trap_receiver_class(self.DEF_TRAP_RECV_ADDR,
                                                 self.DEF_TRAP_RECV_PORT)
        self.mock_object(self.trap_receiver,
                         'alert_rpc_api', self.alert_rpc_api)

    def _get_trap_receiver(self):
        return self.trap_receiver

    @mock.patch('pysnmp.carrier.asyncore.dispatch.AbstractTransportDispatcher'
                '.jobStarted')
    @mock.patch('delfin.db.api.alert_source_get_all')
    @mock.patch('pysnmp.carrier.asyncore.dgram.udp.UdpTransport'
                '.openServerMode', mock.Mock())
    @mock.patch('delfin.alert_manager.trap_receiver.TrapReceiver'
                '._mib_builder', mock.Mock())
    def test_start_success(self, mock_alert_source, mock_dispatcher):
        mock_alert_source.return_value = {}
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.trap_receiver_address = self.DEF_TRAP_RECV_ADDR
        trap_receiver_inst.trap_receiver_port = self.DEF_TRAP_RECV_PORT
        trap_receiver_inst.start()

        # Verify that snmp engine is initialised and transport config is set
        self.assertTrue(trap_receiver_inst.snmp_engine is not None)

    @mock.patch('pysnmp.carrier.asyncore.dispatch.AbstractTransportDispatcher'
                '.jobStarted')
    @mock.patch('delfin.db.api.alert_source_get_all')
    @mock.patch('delfin.alert_manager.trap_receiver.TrapReceiver'
                '._load_snmp_config', fakes.load_config_exception)
    def test_start_with_exception(self, mock_alert_source, mock_dispatcher):
        mock_alert_source.return_value = {}
        trap_receiver_inst = self._get_trap_receiver()

        # Mock load config to raise exception
        self.assertRaisesRegex(ValueError, "Failed to setup for trap listener",
                               trap_receiver_inst.start)

    @mock.patch('pysnmp.carrier.asyncore.dgram.udp.UdpTransport'
                '.openServerMode', mock.Mock())
    def test_add_transport_successful(self):
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        trap_receiver_inst.trap_receiver_address = self.DEF_TRAP_RECV_ADDR
        trap_receiver_inst.trap_receiver_port = self.DEF_TRAP_RECV_PORT
        trap_receiver_inst._add_transport()
        get_transport = config.getTransport(trap_receiver_inst.snmp_engine,
                                            udp.domainName)
        # Verify that snmp engine transport config is set after _add_transport
        self.assertTrue(get_transport is not None)

    def test_add_transport_exception(self):
        trap_receiver_inst = self._get_trap_receiver()

        # Mock exception by not initialising snmp engine
        self.assertRaisesRegex(ValueError,
                               "Port binding failed: Port is in use",
                               trap_receiver_inst._add_transport)

    @mock.patch('pysnmp.carrier.asyncore.dispatch.AbstractTransportDispatcher'
                '.jobStarted')
    @mock.patch('pysnmp.carrier.asyncore.dispatch.AbstractTransportDispatcher'
                '.closeDispatcher')
    @mock.patch('delfin.db.api.alert_source_get_all')
    @mock.patch('pysnmp.carrier.asyncore.dgram.udp.UdpTransport'
                '.openServerMode', mock.Mock())
    @mock.patch('pysnmp.entity.config.addTransport', fakes.mock_add_transport)
    @mock.patch('delfin.alert_manager.trap_receiver.TrapReceiver'
                '._mib_builder', mock.Mock())
    def test_stop_with_snmp_engine(self, mock_alert_source,
                                   mock_close_dispatcher, mock_dispatcher):
        mock_alert_source.return_value = {}
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.trap_receiver_address = self.DEF_TRAP_RECV_ADDR
        trap_receiver_inst.trap_receiver_port = self.DEF_TRAP_RECV_PORT
        trap_receiver_inst.start()
        trap_receiver_inst.stop()

        # Verify that close dispatcher is called during alert manager stop
        self.assertTrue(mock_close_dispatcher.called)

    @mock.patch('pysnmp.carrier.asyncore.dispatch.AbstractTransportDispatcher'
                '.closeDispatcher')
    def test_stop_without_snmp_engine(self, mock_close_dispatcher):
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.stop()

        # Verify that close dispatcher is not called when engine is not
        # initialised
        self.assertFalse(mock_close_dispatcher.called)

    @mock.patch('delfin.cryptor.decode', mock.Mock(return_value='public'))
    @mock.patch('delfin.alert_manager.snmp_validator.SNMPValidator.validate')
    @mock.patch('pysnmp.entity.config.addV1System')
    def test_sync_snmp_config_add_v2_version(self, mock_add_config,
                                             mock_validator):
        ctxt = {}
        alert_config = {'storage_id': 'abcd-1234-5678',
                        'version': 'snmpv2c',
                        'community_string': 'public'}
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        trap_receiver_inst.sync_snmp_config(ctxt,
                                            snmp_config_to_add=alert_config)

        # Verify that config is added to snmp engine
        # Storage_id is internally modified to remove '-' while adding
        mock_add_config.assert_called_once_with(trap_receiver_inst.snmp_engine,
                                                'abcd12345678',
                                                alert_config[
                                                    'community_string'],
                                                contextName=alert_config[
                                                    'community_string'])
        mock_validator.assert_called_once_with(ctxt, alert_config)

    @mock.patch('pysnmp.entity.config.delV1System')
    def test_sync_snmp_config_del_v2_version(self, mock_del_config):
        ctxt = {}
        alert_config = {'storage_id': 'abcd-1234-5678',
                        'version': 'snmpv2c',
                        'community_string': 'public'}
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        trap_receiver_inst.sync_snmp_config(ctxt,
                                            snmp_config_to_del=alert_config)

        # Verify that config is deleted from snmp engine
        # Storage_id is internally modified to remove '-' while deleting
        mock_del_config.assert_called_once_with(trap_receiver_inst.snmp_engine,
                                                'abcd12345678')

    def test_sync_snmp_config_add_invalid_version(self):
        ctxt = {}
        alert_source_config = {'storage_id': 'abcd-1234-5678',
                               'version': 'snmpv4',
                               'community_string': 'public'}
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        self.assertRaisesRegex(exception.InvalidSNMPConfig, "Invalid snmp "
                                                            "version",
                               trap_receiver_inst.sync_snmp_config, ctxt,
                               snmp_config_to_add=alert_source_config)

    @mock.patch('pysnmp.entity.config.addV3User')
    def test_sync_snmp_config_add_v3_version(self, mock_add_config):
        ctxt = {}
        alert_config = fakes.fake_v3_alert_source()
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        trap_receiver_inst.sync_snmp_config(ctxt,
                                            snmp_config_to_add=alert_config)

        # Verify that addV3User to add config to engine
        self.assertTrue(mock_add_config.called)

    @mock.patch('pysnmp.entity.config.delV3User')
    def test_sync_snmp_config_del_v3_version(self, mock_del_config):
        ctxt = {}
        alert_config = fakes.fake_v3_alert_source()
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        trap_receiver_inst.sync_snmp_config(ctxt,
                                            snmp_config_to_add=alert_config)
        trap_receiver_inst.sync_snmp_config(ctxt,
                                            snmp_config_to_del=alert_config)

        # Verify that delV3User to del config from engine
        self.assertTrue(mock_del_config.called)

    def test_sync_snmp_config_invalid_auth_protocol(self):
        ctxt = {}
        alert_source_config = fakes.fake_v3_alert_source()
        alert_source_config['auth_protocol'] = 'invalid_auth'
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        self.assertRaisesRegex(exception.InvalidSNMPConfig, "Invalid "
                                                            "auth_protocol",
                               trap_receiver_inst.sync_snmp_config, ctxt,
                               snmp_config_to_add=alert_source_config)

    def test_sync_snmp_config_invalid_priv_protocol(self):
        ctxt = {}
        alert_source_config = fakes.fake_v3_alert_source()
        alert_source_config['privacy_protocol'] = 'invalid_priv'
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        self.assertRaisesRegex(exception.InvalidSNMPConfig, "Invalid "
                                                            "privacy_protocol",
                               trap_receiver_inst.sync_snmp_config, ctxt,
                               snmp_config_to_add=alert_source_config)

    @mock.patch('pysnmp.entity.config.addV3User')
    @mock.patch('delfin.db.api.alert_source_get_all')
    def test_load_snmp_config(self, mock_alert_source_list, mock_add_config):
        mock_alert_source_list.return_value = fakes.fake_v3_alert_source_list()
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        trap_receiver_inst._load_snmp_config()

        # Verify that config is added to engine
        self.assertTrue(mock_add_config.called)

    @mock.patch('delfin.db.alert_source_get_all')
    def test_get_alert_source_by_host_success(self, mock_alert_source_list):
        # alert_source_config = fakes.fake_v3_alert_source()
        expected_alert_source = {'storage_id': 'abcd-1234-5678',
                                 'version': 'snmpv3',
                                 'engine_id': '800000d30300000e112245',
                                 'username': 'test1',
                                 'auth_key': 'YWJjZDEyMzQ1Njc=',
                                 'auth_protocol': 'HMACMD5',
                                 'privacy_key': 'YWJjZDEyMzQ1Njc=',
                                 'privacy_protocol': 'DES'
                                 }
        mock_alert_source_list.return_value = fakes. \
            fake_v3_alert_source_list_with_one()
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        alert_source = trap_receiver_inst. \
            _get_alert_source_by_host('127.0.0.1')
        self.assertDictEqual(expected_alert_source, alert_source)

    @mock.patch('delfin.db.alert_source_get_all')
    def test_get_alert_source_by_host_without_storage(self,
                                                      mock_alert_source_list):
        # alert_source_config = fakes.fake_v3_alert_source()
        mock_alert_source_list.return_value = fakes.null_alert_source_list()
        trap_receiver_inst = self._get_trap_receiver()
        trap_receiver_inst.snmp_engine = engine.SnmpEngine()
        self.assertRaisesRegex(exception.AlertSourceNotFoundWithHost, "",
                               trap_receiver_inst._get_alert_source_by_host,
                               '127.0.0.1')



from unittest import mock

from delfin import context, exception
from delfin import test
from delfin.db import api as db_api
from delfin.db.sqlalchemy import api, models

ctxt = context.get_admin_context()


class TestSIMDBAPI(test.TestCase):

    @mock.patch('sqlalchemy.create_engine', mock.Mock())
    def test_register_db(self):
        db_api.register_db()

    def test_get_session(self):
        api.get_session()

    def test_get_engine(self):
        api.get_engine()

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_get(self, mock_session):
        fake_storage = {}
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage
        result = db_api.storage_get(ctxt,
                                    'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_update(self, mock_session):
        fake_storage = models.Storage()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage
        result = db_api.storage_update(ctxt,
                                       'c5c91c98-91aa-40e6-85ac-37a1d3b32bda',
                                       fake_storage)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_delete(self, mock_session):
        fake_storage = models.Storage()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage
        result = db_api.storage_delete(ctxt,
                                       'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        assert result is None

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_create(self, mock_session):
        fake_storage = models.Storage()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage
        result = db_api.storage_create(ctxt, fake_storage)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_get_all(self, mock_session):
        fake_storage = []
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage
        result = db_api.storage_get_all(ctxt)
        assert len(result) == 0

        mock_session.return_value.__enter__.return_value.query = fake_storage
        result = db_api.storage_get_all(ctxt, filters={'status': 'Normal'})
        assert len(result) == 0

        result = db_api.storage_get_all(ctxt, limit=1)
        assert len(result) == 0

        result = db_api.storage_get_all(ctxt, offset=3)
        assert len(result) == 0

        result = db_api.storage_get_all(ctxt, sort_dirs=['desc'],
                                        sort_keys=['name'])
        assert len(result) == 0

        self.assertRaises(exception.InvalidInput, api.storage_get_all,
                          ctxt, sort_dirs=['desc', 'asc'],
                          sort_keys=['name'])

        self.assertRaises(exception.InvalidInput, api.storage_get_all,
                          ctxt, sort_dirs=['desc_err'],
                          sort_keys=['name'])

        result = db_api.storage_get_all(ctxt, sort_dirs=['desc'],
                                        sort_keys=['name', 'id'])
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_pool_get(self, mock_session):
        fake_storage_pool = {}
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage_pool
        result = db_api.storage_pool_get(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd')
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_pool_get_all(self, mock_session):
        fake_storage_pool = []
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage_pool
        result = api.storage_pool_get_all(context)
        assert len(result) == 0

        result = db_api.storage_pool_get_all(context,
                                             filters={'status': 'Normal'})
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_pools_update(self, mock_session):
        storage_pools = [{'id': 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd'}]
        mock_session.return_value.__enter__.return_value.query.return_value \
            = storage_pools
        result = db_api.storage_pools_update(context, storage_pools)
        assert len(result) == 1

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_pool_update(self, mock_session):
        values = {'id': 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd'}
        mock_session.return_value.__enter__.return_value.query.return_value \
            = values
        result = db_api.storage_pool_update(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd', values)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_pools_delete(self, mock_session):
        fake_storage_pools = [models.StoragePool().id]
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage_pools
        result = db_api.storage_pools_delete(context, fake_storage_pools)
        assert result is None

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_pools_create(self, mock_session):
        fake_storage_pools = [models.StoragePool()]
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage_pools
        result = db_api.storage_pools_create(context, fake_storage_pools)
        assert len(result) == 1

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_storage_pool_create(self, mock_session):
        fake_storage_pool = models.StoragePool()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_storage_pool
        result = db_api.storage_pool_create(context, fake_storage_pool)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_volume_get(self, mock_session):
        fake_volume = {}
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_volume
        result = db_api.volume_get(ctxt,
                                   'c5c91c98-91aa-40e6-85ac-37a1d3b32bd')
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_volumes_update(self, mock_session):
        volumes = [{'id': 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd'}]
        mock_session.return_value.__enter__.return_value.query.return_value \
            = volumes
        result = db_api.volumes_update(ctxt, volumes)
        assert result is None

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_volume_update(self, mock_session):
        volumes = [{'id': 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd'}]
        mock_session.return_value.__enter__.return_value.query.return_value \
            = volumes
        result = db_api.volume_update(ctxt,
                                      'c5c91c98-91aa-40e6-85ac-37a1d3b32bd',
                                      volumes)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_volumes_delete(self, mock_session):
        fake_volume = ['c5c91c98-91aa-40e6-85ac-37a1d3b32bd']
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_volume
        result = db_api.volumes_delete(ctxt, fake_volume)
        assert result is None

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_volumes_create(self, mock_session):
        fake_volume = [models.Volume()]
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_volume
        result = db_api.volumes_create(ctxt, fake_volume)
        assert len(result) == 1

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_volume_create(self, mock_session):
        fake_volume = models.Volume()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_volume
        result = db_api.volume_create(ctxt, fake_volume)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_volume_get_all(self, mock_session):
        fake_volume = []
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_volume
        result = db_api.volume_get_all(ctxt)
        assert len(result) == 0

        result = db_api.volume_get_all(ctxt, filters={'status': 'Normal'})
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_access_info_get_all(self, mock_session):
        fake_access_info = []
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_access_info
        result = db_api.access_info_get_all(ctxt)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_access_info_get(self, mock_session):
        fake_access_info = models.AccessInfo()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_access_info
        result = db_api.access_info_get(ctxt,
                                        'c5c91c98-91aa-40e6-85ac-37a1d3b32bd')
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_access_info_create(self, mock_session):
        fake_access_info = models.AccessInfo()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_access_info
        result = db_api.access_info_create(ctxt, fake_access_info)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_access_info_update(self, mock_session):
        fake_access_info = models.AccessInfo()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_access_info
        result = db_api.access_info_update(
            ctxt, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd', fake_access_info)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_alert_source_get_all(self, mock_session):
        fake_alert_source = []
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_alert_source
        result = db_api.alert_source_get_all(ctxt)
        assert len(result) == 0

        result = db_api.alert_source_get_all(ctxt,
                                             filters={'status': 'Normal'})
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_alert_source_update(self, mock_session):
        fake_alert_source = models.AlertSource()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_alert_source
        result = db_api.alert_source_update(
            ctxt, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd', fake_alert_source)
        assert len(result) == 0

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_alert_source_delete(self, mock_session):
        fake_alert_source = models.AlertSource()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_alert_source
        result = db_api.alert_source_delete(
            ctxt, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bd')
        assert result is None

    @mock.patch('delfin.db.sqlalchemy.api.get_session')
    def test_alert_source_create(self, mock_session):
        fake_alert_source = models.AlertSource()
        mock_session.return_value.__enter__.return_value.query.return_value \
            = fake_alert_source
        result = db_api.alert_source_create(ctxt, fake_alert_source)
        assert len(result) == 0



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from oslo_utils import importutils

from delfin.common import config # noqa
from delfin import cryptor
from delfin import exception
from delfin.tests.unit.api import fakes


class AlertSourceControllerTestCase(unittest.TestCase):
    ALERT_CONTROLLER_CLASS = 'delfin.api.v1.alert_source.AlertSourceController'

    @mock.patch('delfin.alert_manager.rpcapi.AlertAPI', mock.Mock())
    def _get_alert_controller(self):
        alert_controller_class = importutils.import_class(
            self.ALERT_CONTROLLER_CLASS)
        alert_controller = alert_controller_class()
        return alert_controller

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v3_authpriv_config_create_success(self, mock_alert_source_get,
                                                   mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        expected_alert_source = {'storage_id': 'abcd-1234-5678',
                                 'host': '127.0.0.1',
                                 'community_string': None,
                                 'version': 'snmpv3',
                                 'engine_id': '800000d30300000e112245',
                                 'security_level': None,
                                 'username': 'test1',
                                 'auth_protocol': 'HMACMD5',
                                 'privacy_protocol': 'DES',
                                 'port': 161,
                                 'context_name': "",
                                 'retry_num': 1,
                                 'expiration': 1,
                                 "created_at": '2020-06-15T09:50:31.698956',
                                 "updated_at": '2020-06-15T09:50:31.698956'
                                 }
        mock_alert_source_update.return_value = fakes.fake_v3_alert_source()
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()

        output_alert_source = alert_controller_inst.put(req, fake_storage_id,
                                                        body=body)
        self.assertDictEqual(expected_alert_source, output_alert_source)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v3_config_noauthnopriv_create_success(self,
                                                       mock_alert_source_get,
                                                       mock_alert_source_update
                                                       ):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = fakes. \
            fake_v3_alert_source_noauth_nopriv()
        mock_alert_source_get.return_value = fakes. \
            fake_v3_alert_source_noauth_nopriv()
        expected_alert_source = {'storage_id': 'abcd-1234-5678',
                                 'host': '127.0.0.1',
                                 'community_string': None,
                                 'version': 'snmpv3',
                                 'engine_id': '800000d30300000e112245',
                                 'security_level': 'noAuthnoPriv',
                                 'username': 'test1',
                                 'auth_protocol': None,
                                 'privacy_protocol': None,
                                 'port': 161,
                                 'context_name': "",
                                 'retry_num': 1,
                                 'expiration': 1,
                                 "created_at": '2020-06-15T09:50:31.698956',
                                 "updated_at": '2020-06-15T09:50:31.698956'
                                 }

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['security_level'] = 'noAuthnoPriv'

        output_alert_source = alert_controller_inst.put(req, fake_storage_id,
                                                        body=body)
        self.assertDictEqual(expected_alert_source, output_alert_source)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v3_config_authnopriv_create_success(self,
                                                     mock_alert_source_get,
                                                     mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = fakes. \
            fake_v3_alert_source_auth_nopriv()
        mock_alert_source_get.return_value = fakes. \
            fake_v3_alert_source_auth_nopriv()
        expected_alert_source = {'storage_id': 'abcd-1234-5678',
                                 'host': '127.0.0.1',
                                 'community_string': None,
                                 'version': 'snmpv3',
                                 'engine_id': '800000d30300000e112245',
                                 'security_level': 'authNoPriv',
                                 'username': 'test1',
                                 'auth_protocol': 'HMACMD5',
                                 'privacy_protocol': None,
                                 'port': 161,
                                 'context_name': "",
                                 'retry_num': 1,
                                 'expiration': 1,
                                 "created_at": '2020-06-15T09:50:31.698956',
                                 "updated_at": '2020-06-15T09:50:31.698956'
                                 }
        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['security_level'] = 'authNoPriv'

        output_alert_source = alert_controller_inst.put(req, fake_storage_id,
                                                        body=body)
        self.assertDictEqual(expected_alert_source, output_alert_source)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v2_config_success(self, mock_alert_source_get,
                                   mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        return_v2_alert_source = fakes.fake_v2_alert_source()
        return_v2_alert_source['community_string'] = cryptor.encode(
            return_v2_alert_source['community_string'])
        mock_alert_source_update.return_value = return_v2_alert_source
        mock_alert_source_get.return_value = return_v2_alert_source
        expected_alert_source = {'storage_id': 'abcd-1234-5678',
                                 'host': '127.0.0.1',
                                 'community_string': 'public',
                                 'version': 'snmpv2c',
                                 'engine_id': None,
                                 'security_level': None,
                                 'username': None,
                                 'auth_protocol': None,
                                 'privacy_protocol': None,
                                 'port': 161,
                                 'context_name': "",
                                 'retry_num': 1,
                                 'expiration': 1,
                                 "created_at": '2020-06-15T09:50:31.698956',
                                 "updated_at": '2020-06-15T09:50:31.698956'
                                 }
        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v2_alert_source_config()

        output_alert_source = alert_controller_inst.put(req, fake_storage_id,
                                                        body=body)
        self.assertDictEqual(expected_alert_source, output_alert_source)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_delete')
    @mock.patch('delfin.db.alert_source_get')
    def test_delete_v3_config_success(self, mock_alert_source_get,
                                      mock_alert_source_delete):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_delete.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        alert_controller_inst.delete(req, fake_storage_id)
        self.assertTrue(mock_alert_source_delete.called)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_delete')
    @mock.patch('delfin.db.alert_source_get',
                fakes.alert_source_get_exception)
    def test_delete_v3_config_failure(self, mock_alert_source_delete):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_delete.return_value = {}

        alert_controller_inst = self._get_alert_controller()
        self.assertRaisesRegex(exception.AlertSourceNotFound, "Alert source "
                                                              "for storage "
                                                              "abcd-1234-5678 "
                                                              "could not be "
                                                              "found",
                               alert_controller_inst.delete, req,
                               fake_storage_id)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_get')
    def test_show_v3_config(self, mock_alert_source_get):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()
        expected_alert_source = {'storage_id': 'abcd-1234-5678',
                                 'host': '127.0.0.1',
                                 'community_string': None,
                                 'version': 'snmpv3',
                                 'engine_id': '800000d30300000e112245',
                                 'security_level': None,
                                 'username': 'test1',
                                 'auth_protocol': 'HMACMD5',
                                 'privacy_protocol': 'DES',
                                 'port': 161,
                                 'context_name': "",
                                 'retry_num': 1,
                                 'expiration': 1,
                                 "created_at": '2020-06-15T09:50:31.698956',
                                 "updated_at": '2020-06-15T09:50:31.698956'
                                 }
        alert_controller_inst = self._get_alert_controller()
        output_alert_source = alert_controller_inst.show(req, fake_storage_id)
        self.assertDictEqual(expected_alert_source, output_alert_source)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v3_authpriv_no_priv_key(self,
                                         mock_alert_source_get,
                                         mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['security_level'] = 'authPriv'
        body['privacy_key'] = ''

        self.assertRaisesRegex(exception.InvalidInput, "Invalid input for "
                                                       "field/attribute "
                                                       "privacy_key",
                               alert_controller_inst.put, req, fake_storage_id,
                               body=body)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v3_authpriv_no_priv_protocol(self,
                                              mock_alert_source_get,
                                              mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['security_level'] = 'authPriv'
        body['privacy_protocol'] = ''

        self.assertRaisesRegex(exception.InvalidInput, "Invalid input for "
                                                       "field/attribute "
                                                       "privacy_protocol",
                               alert_controller_inst.put, req, fake_storage_id,
                               body=body)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v3_authnopriv_no_auth_protocol(self,
                                                mock_alert_source_get,
                                                mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['security_level'] = 'authNoPriv'
        body['auth_protocol'] = ''

        self.assertRaisesRegex(exception.InvalidInput, "Invalid input for "
                                                       "field/attribute "
                                                       "auth_protocol",
                               alert_controller_inst.put, req, fake_storage_id,
                               body=body)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_v3_authnopriv_no_auth_key(self,
                                           mock_alert_source_get,
                                           mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['security_level'] = 'authNoPriv'
        body['auth_key'] = ''

        self.assertRaisesRegex(exception.InvalidInput, "Invalid input for "
                                                       "field/attribute "
                                                       "auth_key",
                               alert_controller_inst.put, req, fake_storage_id,
                               body=body)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_without_username(self, mock_alert_source_get,
                                  mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['username'] = ''

        self.assertRaisesRegex(exception.InvalidInput, "Invalid input "
                                                       "received. Invalid "
                                                       "input for "
                                                       "field/attribute "
                                                       "username.",
                               alert_controller_inst.put, req, fake_storage_id,
                               body=body)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_without_engine_id(self, mock_alert_source_get,
                                   mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v3_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['engine_id'] = ''

        self.assertRaisesRegex(exception.InvalidInput, "Invalid input "
                                                       "received. Invalid "
                                                       "input for "
                                                       "field/attribute "
                                                       "engine_id..",
                               alert_controller_inst.put, req, fake_storage_id,
                               body=body)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    def test_put_without_community_str(self, mock_alert_source_get,
                                       mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = {}
        mock_alert_source_get.return_value = fakes.fake_v2_alert_source()

        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v2_alert_source_config()
        body['community_string'] = ''

        self.assertRaisesRegex(exception.InvalidInput, "Invalid input "
                                                       "received. Invalid "
                                                       "input for "
                                                       "field/attribute "
                                                       "community_string.",
                               alert_controller_inst.put, req, fake_storage_id,
                               body=body)

    @mock.patch('delfin.db.storage_get', mock.Mock())
    @mock.patch('delfin.db.alert_source_update')
    @mock.patch('delfin.db.alert_source_get')
    @mock.patch('pysnmp.entity.rfc3413.oneliner.cmdgen.CommandGenerator'
                '.getCmd', fakes.fake_getcmd_success)
    def test_put_v3_snmp_validation_success(self,
                                            mock_alert_source_get,
                                            mock_alert_source_update):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alert-source')
        fake_storage_id = 'abcd-1234-5678'
        mock_alert_source_update.return_value = fakes. \
            fake_v3_alert_source_auth_nopriv()
        mock_alert_source_get.return_value = fakes. \
            fake_v3_alert_source_auth_nopriv()
        expected_alert_source = {'storage_id': 'abcd-1234-5678',
                                 'host': '127.0.0.1',
                                 'community_string': None,
                                 'version': 'snmpv3',
                                 'engine_id': '800000d30300000e112245',
                                 'security_level': 'authNoPriv',
                                 'username': 'test1',
                                 'auth_protocol': 'HMACMD5',
                                 'privacy_protocol': None,
                                 'port': 161,
                                 'context_name': "",
                                 'retry_num': 1,
                                 'expiration': 1,
                                 "created_at": '2020-06-15T09:50:31.698956',
                                 "updated_at": '2020-06-15T09:50:31.698956'
                                 }
        alert_controller_inst = self._get_alert_controller()
        body = fakes.fake_v3_alert_source_config()
        body['security_level'] = 'authNoPriv'

        output_alert_source = alert_controller_inst.put(req, fake_storage_id,
                                                        body=body)
        self.assertDictEqual(expected_alert_source, output_alert_source)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six
import abc


@six.add_metaclass(abc.ABCMeta)
class StorageDriver(object):

    def __init__(self, **kwargs):
        """
        :param kwargs:  A dictionary, include access information. Pay
            attention that it's not safe to save username and password
            in memory, so suggest each driver use them to get session
            instead of save them in memory directly.
        """
        self.storage_id = kwargs.get('storage_id', None)

    @abc.abstractmethod
    def reset_connection(self, context, **kwargs):
        """ Reset connection with backend with new args """
        pass

    @abc.abstractmethod
    def get_storage(self, context):
        """Get storage device information from storage system"""
        pass

    @abc.abstractmethod
    def list_storage_pools(self, context):
        """List all storage pools from storage system."""
        pass

    @abc.abstractmethod
    def list_volumes(self, context):
        """List all storage volumes from storage system."""
        pass

    @abc.abstractmethod
    def add_trap_config(self, context, trap_config):
        """Config the trap receiver in storage system."""
        pass

    @abc.abstractmethod
    def remove_trap_config(self, context, trap_config):
        """Remove trap receiver configuration from storage system."""
        pass

    @abc.abstractmethod
    def parse_alert(self, context, alert):
        """Parse alert data got from snmp trap server."""

        """
        Alert Model	Description
        *****Filled from driver side ***********************
        alert_id	Unique identification for a given alert type
        alert_name	Unique name for a given alert type
        severity	Severity of the alert
        category	Category of alert generated
        type	Type of the alert generated
        sequence_number	Sequence number for the alert, uniquely identifies a
                                  given alert instance used for
                                  clearing the alert
        occur_time	Time at which alert is generated from device in epoch
                    format
        description	Possible cause description or other details about
                                the alert
        recovery_advice	Some suggestion for handling the given alert
        resource_type	Resource type of device/source generating alert
        location	Detailed info about the tracing the alerting device such as
                    slot, rack, component, parts etc
        *****************************************************
        """

        pass

    def list_alerts(self, context):
        """List all current alerts from storage system."""
        pass

    @abc.abstractmethod
    def clear_alert(self, context, sequence_number):
        """Clear alert from storage system."""
        pass



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log
from delfin.exporter import base_exporter

LOG = log.getLogger(__name__)


class AlertExporterExample(base_exporter.BaseExporter):
    def dispatch(self, ctxt, data):
        LOG.info("AlertExporterExample, report data: %s" % data)


class PerformanceExporterExample(base_exporter.BaseExporter):
    def dispatch(self, ctxt, data):
        LOG.info("PerformanceExporterExample, report data: %s" % data)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class OidMapper(object):
    """Functions/attributes for oid to alert info mapper"""

    # Map to translate trap oid strings to oid names
    OID_MAP = {
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.1": "hwIsmReportingAlarmNodeCode",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.2": "hwIsmReportingAlarmLocationInfo",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.3": "hwIsmReportingAlarmRestoreAdvice",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.4": "hwIsmReportingAlarmFaultTitle",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.5": "hwIsmReportingAlarmFaultType",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.6": "hwIsmReportingAlarmFaultLevel",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.7": "hwIsmReportingAlarmAlarmID",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.8": "hwIsmReportingAlarmFaultTime",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.9": "hwIsmReportingAlarmSerialNo",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.10": "hwIsmReportingAlarmAdditionInfo",
        "1.3.6.1.4.1.2011.2.91.10.3.1.1.11": "hwIsmReportingAlarmFaultCategory"
    }

    def __init__(self):
        pass

    def map_oids(self, alert):
        """Translate oids using static map."""
        alert_model = dict()

        for attr in alert:
            # Remove the instance number at the end of oid before mapping
            oid_str = attr.rsplit('.', 1)[0]
            key = self.OID_MAP.get(oid_str, None)
            alert_model[key] = alert[attr]

        return alert_model



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock


from delfin import db
from delfin import exception
from delfin import test
from delfin.api.v1.storage_pools import StoragePoolController
from delfin.tests.unit.api import fakes


class TestStoragePoolController(test.TestCase):

    def setUp(self):
        super(TestStoragePoolController, self).setUp()
        self.controller = StoragePoolController()

    def test_list(self):
        self.mock_object(
            db, 'storage_pool_get_all',
            fakes.fake_storage_pool_get_all)
        req = fakes.HTTPRequest.blank('/storage-pools')

        res_dict = self.controller.index(req)

        expctd_dict = {
            "storage_pools": [
                {
                    "created_at": "2020-06-10T07:17:08.707356",
                    "updated_at": "2020-06-10T07:17:08.707356",
                    "id": "14155a1f-f053-4ccb-a846-ed67e4387428",
                    "name": "SRP_1",
                    "storage_id": '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
                    "native_storage_pool_id": "SRP_1",
                    "description": "fake storage Pool",
                    "status": "normal",
                    "storage_type": "block",
                    "total_capacity": 26300318136401,
                    "used_capacity": 19054536509358,
                    "free_capacity": 7245781627043,
                    'subscribed_capacity': 219902325555200
                }
            ]
        }
        self.assertDictEqual(expctd_dict, res_dict)

    def test_list_with_filter(self):
        self.mock_object(
            db, 'storage_pool_get_all',
            fakes.fake_storage_pool_get_all)
        req = fakes.HTTPRequest.blank(
            '/storage-pools/'
            '?storage_id=12c2d52f-01bc-41f5-b73f-7abf6f38a2a6'
            '&sort=name:asc&wrongfilter=remove')
        res_dict = self.controller.index(req)
        expctd_dict = {
            "storage_pools": [
                {
                    "created_at": "2020-06-10T07:17:08.707356",
                    "updated_at": "2020-06-10T07:17:08.707356",
                    "id": "14155a1f-f053-4ccb-a846-ed67e4387428",
                    "name": "SRP_1",
                    "storage_id": '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
                    "native_storage_pool_id": "SRP_1",
                    "description": "fake storage Pool",
                    "status": "normal",
                    "storage_type": "block",
                    "total_capacity": 26300318136401,
                    "used_capacity": 19054536509358,
                    "free_capacity": 7245781627043,
                    'subscribed_capacity': 219902325555200
                }
            ]
        }

        self.assertDictEqual(expctd_dict, res_dict)

    def test_show(self):
        self.mock_object(
            db, 'storage_pool_get',
            fakes.fake_storage_pool_show)
        req = fakes.HTTPRequest.blank(
            '/storage-pools/14155a1f-f053-4ccb-a846-ed67e4387428')

        res_dict = self.controller.show(
            req, '14155a1f-f053-4ccb-a846-ed67e4387428')
        expctd_dict = {
            "created_at": "2020-06-10T07:17:08.707356",
            "updated_at": "2020-06-10T07:17:08.707356",
            "id": "14155a1f-f053-4ccb-a846-ed67e4387428",
            "name": "SRP_1",
            "storage_id": '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
            "native_storage_pool_id": "SRP_1",
            "description": "fake storage Pool",
            "status": "normal",
            "storage_type": "block",
            "total_capacity": 26300318136401,
            "used_capacity": 19054536509358,
            "free_capacity": 7245781627043,
            'subscribed_capacity': 219902325555200
        }
        self.assertDictEqual(expctd_dict, res_dict)

    def test_show_with_invalid_id(self):
        self.mock_object(
            db, 'storage_pool_get',
            mock.Mock(side_effect=exception.StoragePoolNotFound('fake_id')))
        req = fakes.HTTPRequest.blank('/storage-pools/fake_id')
        self.assertRaises(exception.StoragePoolNotFound,
                          self.controller.show,
                          req, 'fake_id')



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log
from oslo_utils import units
from delfin.common import constants
from delfin.drivers.dell_emc.vmax.alert_handler import snmp_alerts
from delfin.drivers.dell_emc.vmax.alert_handler import unisphere_alerts
from delfin.drivers.dell_emc.vmax import client
from delfin.drivers import driver

LOG = log.getLogger(__name__)


class VMAXStorageDriver(driver.StorageDriver):
    """VMAXStorageDriver implement the DELL EMC Storage driver,
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.client = client.VMAXClient(**kwargs)
        self.client.init_connection(kwargs)

    def reset_connection(self, context, **kwargs):
        self.client.reset_connection(**kwargs)

    def get_storage(self, context):
        # Get the VMAX model
        array_details = self.client.get_array_details()
        model = array_details['model']
        ucode = array_details['ucode']
        display_name = array_details['display_name']

        # Get Storage details for capacity info
        storg_info = self.client.get_storage_capacity()
        system_capacity = storg_info['system_capacity']
        physical_capacity = storg_info['physicalCapacity']
        total_cap = system_capacity.get('usable_total_tb')
        used_cap = system_capacity.get('usable_used_tb')
        subscribed_cap = system_capacity.get('subscribed_total_tb')
        total_raw = physical_capacity.get('total_capacity_gb')
        free_cap = total_cap - used_cap

        storage = {
            # Unisphere Rest API do not provide Array name .
            # Generate  name  by combining model and symmetrixId
            'name': display_name,
            'vendor': 'Dell EMC',
            'description': '',
            'model': model,
            'firmware_version': ucode,
            'status': constants.StorageStatus.NORMAL,
            'serial_number': self.client.array_id,
            'location': '',
            'total_capacity': int(total_cap * units.Ti),
            'used_capacity': int(used_cap * units.Ti),
            'free_capacity': int(free_cap * units.Ti),
            'raw_capacity': int(total_raw * units.Gi),
            'subscribed_capacity': int(subscribed_cap * units.Ti)
        }
        LOG.info("get_storage(), successfully retrieved storage details")
        return storage

    def list_storage_pools(self, context):
        return self.client.list_storage_pools(self.storage_id)

    def list_volumes(self, context):
        return self.client.list_volumes(self.storage_id)

    def add_trap_config(self, context, trap_config):
        pass

    def remove_trap_config(self, context, trap_config):
        pass

    def parse_alert(self, context, alert):
        return snmp_alerts.AlertHandler().parse_alert(context, alert)

    def clear_alert(self, context, sequence_number):
        return self.client.clear_alert(sequence_number)

    def list_alerts(self, context):
        alert_list = self.client.list_alerts()
        alert_model_list = unisphere_alerts.AlertHandler()\
            .parse_queried_alerts(alert_list)
        return alert_model_list



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log
from oslo_utils import units

from delfin import exception
from delfin.common import constants
from delfin.drivers.dell_emc.vmax import rest

LOG = log.getLogger(__name__)

EMBEDDED_UNISPHERE_ARRAY_COUNT = 1


class VMAXClient(object):
    """ Client class for communicating with VMAX storage """

    def __init__(self, **kwargs):
        self.uni_version = None
        self.array_id = None
        rest_access = kwargs.get('rest')
        if rest_access is None:
            raise exception.InvalidInput('Input rest_access is missing')
        self.rest_host = rest_access.get('host')
        self.rest_port = rest_access.get('port')
        self.rest_username = rest_access.get('username')
        self.rest_password = rest_access.get('password')
        self.rest = rest.VMaxRest()
        self.rest.set_rest_credentials(rest_access)
        self.reset_connection(**kwargs)

    def reset_connection(self, **kwargs):
        """ Reset connection to VMAX storage with new configs """
        self.rest.verify = kwargs.get('verify', False)
        self.rest.establish_rest_session()

    def init_connection(self, access_info):
        """ Given the access_info get a connection to VMAX storage """
        try:
            ver, self.uni_version = self.rest.get_uni_version()
            LOG.info('Connected to Unisphere Version: {0}'.format(ver))
        except exception.InvalidCredential as e:
            msg = "Failed to connect VMAX. Reason: {}".format(e.msg)
            LOG.error(msg)
            raise e
        except Exception as err:
            msg = ("Failed to connect to VMAX. Host or Port is not correct: "
                   "{}".format(err))
            LOG.error(msg)
            raise exception.HTTPConnectionTimeout(msg)

        if not self.uni_version:
            msg = "Invalid input. Failed to get vmax unisphere version"
            raise exception.InvalidInput(msg)

        self.array_id = access_info.get('extra_attributes', {}). \
            get('array_id', None)

        try:
            # Get array details from unisphere
            array = self.rest.get_array_detail(version=self.uni_version)
            if not array:
                msg = "Failed to get array details"
                raise exception.InvalidInput(msg)

            if len(array['symmetrixId']) == EMBEDDED_UNISPHERE_ARRAY_COUNT:
                if not self.array_id:
                    self.array_id = array['symmetrixId'][0]
                elif self.array_id != array['symmetrixId'][0]:
                    msg = "Invalid array_id. Expected id: {}". \
                        format(array['symmetrixId'])
                    raise exception.InvalidInput(msg)
        except Exception as err:
            msg = "Failed to get array details from VMAX: {}".format(err)
            raise exception.StorageBackendException(msg)

        if not self.array_id:
            msg = "Input array_id is missing. Supported ids: {}". \
                format(array['symmetrixId'])
            raise exception.InvalidInput(msg)

    def get_array_details(self):
        try:
            # Get the VMAX array properties
            return self.rest.get_vmax_array_details(version=self.uni_version,
                                                    array=self.array_id)
        except Exception as err:
            msg = "Failed to get array details from VMAX: {}".format(err)
            LOG.error(msg)
            raise exception.StorageBackendException(msg)

    def get_storage_capacity(self):
        try:
            storage_info = self.rest.get_system_capacity(
                self.array_id, self.uni_version)
            return storage_info
        except Exception as err:
            msg = "Failed to get capacity from VMAX: {}".format(err)
            LOG.error(msg)
            raise exception.StorageBackendException(msg)

    def list_storage_pools(self, storage_id):

        try:
            # Get list of SRP pool names
            pools = self.rest.get_srp_by_name(
                self.array_id, self.uni_version, srp='')['srpId']

            pool_list = []
            for pool in pools:
                pool_info = self.rest.get_srp_by_name(
                    self.array_id, self.uni_version, srp=pool)

                srp_cap = pool_info['srp_capacity']
                total_cap = srp_cap['usable_total_tb'] * units.Ti
                used_cap = srp_cap['usable_used_tb'] * units.Ti
                subscribed_cap = srp_cap['subscribed_total_tb'] * units.Ti
                p = {
                    "name": pool,
                    "storage_id": storage_id,
                    "native_storage_pool_id": pool_info["srpId"],
                    "description": "Dell EMC VMAX Pool",
                    "status": constants.StoragePoolStatus.NORMAL,
                    "storage_type": constants.StorageType.BLOCK,
                    "total_capacity": int(total_cap),
                    "used_capacity": int(used_cap),
                    "free_capacity": int(total_cap - used_cap),
                    "subscribed_capacity": int(subscribed_cap),
                }
                pool_list.append(p)

            return pool_list

        except Exception as err:
            msg = "Failed to get pool metrics from VMAX: {}".format(err)
            LOG.error(msg)
            raise exception.StorageBackendException(msg)

    def list_volumes(self, storage_id):

        try:
            # Get default SRPs assigned for the array
            default_srps = self.rest.get_default_srps(
                self.array_id, version=self.uni_version)
            # List all volumes except data volumes
            volumes = self.rest.get_volume_list(
                self.array_id, version=self.uni_version,
                params={'data_volume': 'false'})

            # TODO: Update constants.VolumeStatus to make mapping more precise
            switcher = {
                'Ready': constants.VolumeStatus.AVAILABLE,
                'Not Ready': constants.VolumeStatus.ERROR,
                'Mixed': constants.VolumeStatus.ERROR,
                'Write Disabled': constants.VolumeStatus.ERROR,
                'N/A': constants.VolumeStatus.ERROR,
            }

            volume_list = []
            for volume in volumes:
                # Get volume details
                vol = self.rest.get_volume(self.array_id,
                                           self.uni_version, volume)

                emulation_type = vol['emulation']
                total_cap = vol['cap_mb'] * units.Mi
                used_cap = (total_cap * vol['allocated_percent']) / 100.0
                free_cap = total_cap - used_cap

                status = switcher.get(vol['status'],
                                      constants.VolumeStatus.ERROR)

                description = "Dell EMC VMAX volume"
                if vol['type'] == 'TDEV':
                    description = "Dell EMC VMAX 'thin device' volume"

                v = {
                    "name": volume,
                    "storage_id": storage_id,
                    "description": description,
                    "status": status,
                    "native_volume_id": vol['volumeId'],
                    "wwn": vol['wwn'],
                    "type": constants.VolumeType.THIN,
                    "total_capacity": int(total_cap),
                    "used_capacity": int(used_cap),
                    "free_capacity": int(free_cap),
                }

                if vol['num_of_storage_groups'] == 1:
                    sg = vol['storageGroupId'][0]
                    sg_info = self.rest.get_storage_group(
                        self.array_id, self.uni_version, sg)
                    v['native_storage_pool_id'] = sg_info['srp']
                    v['compressed'] = sg_info['compression']
                else:
                    v['native_storage_pool_id'] = default_srps[emulation_type]

                volume_list.append(v)

            return volume_list

        except Exception as err:
            msg = "Failed to get list volumes from VMAX: {}".format(err)
            LOG.error(msg)
            raise exception.StorageBackendException(msg)

    def list_alerts(self):
        """Get all alerts from an array."""
        return self.rest.get_alerts(version=self.uni_version,
                                    array=self.array_id)

    def clear_alert(self, sequence_number):
        """Clear alert for given sequence number."""
        return self.rest.clear_alert(sequence_number, version=self.uni_version,
                                     array=self.array_id)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin.api.validation import parameter_types

create = {
    'type': 'object',
    'properties': {
        'vendor': {'type': 'string', 'minLength': 1, 'maxLength': 255},
        'model': {'type': 'string', 'minLength': 1, 'maxLength': 255},
        'rest': {
            'type': 'object',
            'properties': {
                'host': parameter_types.hostname_or_ip_address,
                'port': parameter_types.tcp_udp_port,
                'username': {'type': 'string', 'minLength': 1,
                             'maxLength': 255},
                'password': {'type': 'string', 'minLength': 1,
                             'maxLength': 255}
            },
            'required': ['host', 'port', 'username', 'password'],
            'additionalProperties': False
        },
        'ssh': {
            'type': 'object',
            'properties': {
                'host': parameter_types.hostname_or_ip_address,
                'port': parameter_types.tcp_udp_port,
                'username': {'type': 'string', 'minLength': 1,
                             'maxLength': 255},
                'password': {'type': 'string', 'minLength': 1,
                             'maxLength': 255},
                'pub_key': {'type': 'string', 'minLength': 1,
                            'maxLength': 4096},
                'pub_key_type': parameter_types.host_key_type
            },
            'required': ['host', 'port', 'username', 'password', 'pub_key'],
            'additionalProperties': False
        },
        'extra_attributes': {
            'type': 'object',
            'patternProperties': {
                '^[a-zA-Z0-9-_:. ]{1,255}$': {
                    'type': 'string', 'maxLength': 255
                }
            }
        }
    },
    'required': ['vendor', 'model'],
    'anyOf': [
        {'required': ['rest']},
        {'required': ['ssh']}
    ],
    'additionalProperties': False
}



# Copyright 2020 The SODA Authors.
# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2011 Piston Cloud Computing, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
SQLAlchemy models for Delfin  data.
"""

from oslo_config import cfg
from oslo_db.sqlalchemy import models
from oslo_db.sqlalchemy.types import JsonEncodedDict
from sqlalchemy import Column, Integer, String, Boolean, BigInteger, DateTime
from sqlalchemy.ext.declarative import declarative_base

from delfin.common import constants

CONF = cfg.CONF
BASE = declarative_base()


class DelfinBase(models.ModelBase,
                 models.TimestampMixin):
    """Base class for Delfin Models."""
    __table_args__ = {'mysql_engine': 'InnoDB'}
    metadata = None

    def to_dict(self):
        model_dict = {}
        for k, v in self.items():
            if not issubclass(type(v), DelfinBase):
                model_dict[k] = v
        return model_dict


class AccessInfo(BASE, DelfinBase):
    """Represent access info required for storage accessing."""
    __tablename__ = "access_info"
    storage_id = Column(String(36), primary_key=True)
    vendor = Column(String(255))
    model = Column(String(255))
    rest = Column(JsonEncodedDict)
    ssh = Column(JsonEncodedDict)
    extra_attributes = Column(JsonEncodedDict)


class Storage(BASE, DelfinBase):
    """Represents a storage object."""

    __tablename__ = 'storages'
    id = Column(String(36), primary_key=True)
    name = Column(String(255))
    vendor = Column(String(255))
    description = Column(String(255))
    model = Column(String(255))
    status = Column(String(255))
    serial_number = Column(String(255))
    firmware_version = Column(String(255))
    location = Column(String(255))
    total_capacity = Column(BigInteger)
    used_capacity = Column(BigInteger)
    free_capacity = Column(BigInteger)
    raw_capacity = Column(BigInteger)
    subscribed_capacity = Column(BigInteger)
    sync_status = Column(Integer, default=constants.SyncStatus.SYNCED)
    deleted_at = Column(DateTime)
    deleted = Column(Boolean, default=False)


class Volume(BASE, DelfinBase):
    """Represents a volume object."""
    __tablename__ = 'volumes'
    id = Column(String(36), primary_key=True)
    name = Column(String(255))
    storage_id = Column(String(36))
    native_storage_pool_id = Column(String(255))
    description = Column(String(255))
    status = Column(String(255))
    native_volume_id = Column(String(255))
    wwn = Column(String(255))
    type = Column(String(255))
    total_capacity = Column(BigInteger)
    used_capacity = Column(BigInteger)
    free_capacity = Column(BigInteger)
    compressed = Column(Boolean)
    deduplicated = Column(Boolean)


class StoragePool(BASE, DelfinBase):
    """Represents a storage_pool object."""
    __tablename__ = 'storage_pools'
    id = Column(String(36), primary_key=True)
    name = Column(String(255))
    storage_id = Column(String(36))
    native_storage_pool_id = Column(String(255))
    description = Column(String(255))
    status = Column(String(255))
    storage_type = Column(String(255))
    total_capacity = Column(BigInteger)
    used_capacity = Column(BigInteger)
    free_capacity = Column(BigInteger)
    subscribed_capacity = Column(BigInteger)


class Disk(BASE, DelfinBase):
    """Represents a disk object."""
    __tablename__ = 'disks'
    id = Column(String(36), primary_key=True)
    name = Column(String(255))
    status = Column(String(255))
    vendor = Column(String(255))
    native_disk_id = Column(String(255))
    serial_number = Column(String(255))
    model = Column(String(255))
    media_type = Column(String(255))
    capacity = Column(BigInteger)


class AlertSource(BASE, DelfinBase):
    """Represents an alert source configuration."""
    __tablename__ = 'alert_source'
    storage_id = Column(String(36), primary_key=True)
    host = Column(String(255))
    version = Column(String(255))
    community_string = Column(String(255))
    username = Column(String(255))
    security_level = Column(String(255))
    auth_key = Column(String(255))
    auth_protocol = Column(String(255))
    privacy_protocol = Column(String(255))
    privacy_key = Column(String(255))
    engine_id = Column(String(255))
    port = Column(Integer)
    context_name = Column(String(255))
    retry_num = Column(Integer)
    expiration = Column(Integer)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from delfin import context
from delfin.drivers import driver
from delfin.drivers.hpe.hpe_3par import alert_handler
from delfin.drivers.hpe.hpe_3par import component_handler
from delfin.drivers.hpe.hpe_3par import rest_handler
from delfin.drivers.hpe.hpe_3par import ssh_handler
from delfin.drivers.utils.rest_client import RestClient
from delfin.drivers.utils.ssh_client import SSHClient

LOG = log.getLogger(__name__)


# Hpe3parStor Driver
class Hpe3parStorDriver(driver.StorageDriver):
    """Hpe3parStorDriver implement Hpe 3par Stor driver,
    """

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        # init rest client
        self.restclient = RestClient(**kwargs)
        self.resthanlder = rest_handler.RestHandler(self.restclient)
        self.resthanlder.login()
        # init ssh client
        self.sshclient = SSHClient(**kwargs)
        self.sshhanlder = ssh_handler.SSHHandler(self.sshclient)
        self.version = self.sshhanlder.login(context)
        # init component handler
        self.comhandler = component_handler.ComponentHandler(
            resthanlder=self.resthanlder, sshhanlder=self.sshhanlder)
        # init component handler
        self.alert_handler = alert_handler.AlertHandler(
            resthanlder=self.resthanlder, sshhanlder=self.sshhanlder)

    def reset_connection(self, context, **kwargs):
        self.resthanlder.logout()
        self.restclient.verify = kwargs.get('verify', False)
        self.resthanlder.login()

    def get_storage(self, context):
        # get storage info
        return self.comhandler.get_storage(context)

    def list_storage_pools(self, context):
        # Get list of Hpe3parStor pool details
        self.comhandler.set_storage_id(self.storage_id)
        return self.comhandler.list_storage_pools(context)

    def list_volumes(self, context):
        self.comhandler.set_storage_id(self.storage_id)
        return self.comhandler.list_volumes(context)

    def list_alerts(self, context):
        # Get list of Hpe3parStor alerts
        return self.alert_handler.list_alerts(context)

    def add_trap_config(self, context, trap_config):
        pass

    def remove_trap_config(self, context, trap_config):
        pass

    def parse_alert(self, context, alert):
        return self.alert_handler.parse_alert(context, alert)

    def clear_alert(self, context, alert):
        return self.alert_handler.clear_alert(context, alert)



# Copyright 2020 The SODA Authors.
# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright (c) 2014 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Implementation of SQLAlchemy backend."""

import sys

import six
import sqlalchemy
from oslo_config import cfg
from oslo_db import options as db_options
from oslo_db.sqlalchemy import session
from oslo_db.sqlalchemy import utils as db_utils
from oslo_log import log
from oslo_utils import uuidutils, timeutils
from sqlalchemy import create_engine

from delfin import exception
from delfin.common import sqlalchemyutils
from delfin.db.sqlalchemy import models
from delfin.db.sqlalchemy.models import Storage, AccessInfo
from delfin.i18n import _

CONF = cfg.CONF
LOG = log.getLogger(__name__)
_FACADE = None

_DEFAULT_SQL_CONNECTION = 'sqlite:///'
db_options.set_defaults(cfg.CONF,
                        connection=_DEFAULT_SQL_CONNECTION)


def apply_sorting(model, query, sort_key, sort_dir):
    if sort_dir.lower() not in ('desc', 'asc'):
        msg = (("Wrong sorting data provided: sort key is '%(sort_key)s' "
                "and sort order is '%(sort_dir)s'.") %
               {"sort_key": sort_key, "sort_dir": sort_dir})
        raise exception.InvalidInput(msg)

    sort_attr = getattr(model, sort_key)
    sort_method = getattr(sort_attr, sort_dir.lower())
    return query.order_by(sort_method())


def get_engine():
    facade = _create_facade_lazily()
    return facade.get_engine()


def get_session(**kwargs):
    facade = _create_facade_lazily()
    return facade.get_session(**kwargs)


def _create_facade_lazily():
    global _FACADE
    if _FACADE is None:
        _FACADE = session.EngineFacade.from_config(cfg.CONF)
    return _FACADE


def get_backend():
    """The backend is this module itself."""
    return sys.modules[__name__]


def register_db():
    """Create database and tables."""
    models = (Storage,
              AccessInfo
              )
    engine = create_engine(CONF.database.connection, echo=False)
    for model in models:
        model.metadata.create_all(engine)


def _process_model_like_filter(model, query, filters):
    """Applies regex expression filtering to a query.

    :param model: model to apply filters to
    :param query: query to apply filters to
    :param filters: dictionary of filters with regex values
    :returns: the updated query.
    """
    if query is None:
        return query

    for key in sorted(filters):
        column_attr = getattr(model, key)
        if 'property' == type(column_attr).__name__:
            continue
        value = filters[key]
        if not (isinstance(value, (six.string_types, int))):
            continue
        query = query.filter(
            column_attr.op('LIKE')(u'%%%s%%' % value))
    return query


def apply_like_filters(model):
    def decorator_filters(process_exact_filters):
        def _decorator(query, filters):
            exact_filters = filters.copy()
            regex_filters = {}
            for key, value in filters.items():
                # NOTE(tommylikehu): For inexact match, the filter keys
                # are in the format of 'key~=value'
                if key.endswith('~'):
                    exact_filters.pop(key)
                    regex_filters[key.rstrip('~')] = value
            query = process_exact_filters(query, exact_filters)
            return _process_model_like_filter(model, query, regex_filters)

        return _decorator

    return decorator_filters


def is_valid_model_filters(model, filters, exclude_list=None):
    """Return True if filter values exist on the model

    :param model: a Delfin model
    :param filters: dictionary of filters
    """
    for key in filters.keys():
        if exclude_list and key in exclude_list:
            continue
        if key == 'metadata':
            if not isinstance(filters[key], dict):
                LOG.debug("Metadata filter value is not valid dictionary")
                return False
            continue
        try:
            key = key.rstrip('~')
            getattr(model, key)
        except AttributeError:
            LOG.debug("'%s' filter key is not valid.", key)
            return False
    return True


def access_info_create(context, values):
    """Create a storage access information."""
    if not values.get('storage_id'):
        values['storage_id'] = uuidutils.generate_uuid()

    access_info_ref = models.AccessInfo()
    access_info_ref.update(values)

    session = get_session()
    with session.begin():
        session.add(access_info_ref)

    return _access_info_get(context,
                            access_info_ref['storage_id'],
                            session=session)


def access_info_update(context, storage_id, values):
    """Update a storage access information with the values dictionary."""
    session = get_session()
    with session.begin():
        _access_info_get(context, storage_id, session).update(values)
        return _access_info_get(context, storage_id, session)


def access_info_delete(context, storage_id):
    """Delete a storage access information."""
    _access_info_get_query(context). \
        filter_by(storage_id=storage_id).delete()


def access_info_get(context, storage_id):
    """Get a storage access information."""
    return _access_info_get(context, storage_id)


def _access_info_get(context, storage_id, session=None):
    result = (_access_info_get_query(context, session=session)
              .filter_by(storage_id=storage_id)
              .first())

    if not result:
        raise exception.AccessInfoNotFound(storage_id)

    return result


def _access_info_get_query(context, session=None):
    return model_query(context, models.AccessInfo, session=session)


def access_info_get_all(context, marker=None, limit=None, sort_keys=None,
                        sort_dirs=None, filters=None, offset=None):
    """Retrieves all storage access information."""
    session = get_session()
    with session.begin():
        query = _generate_paginate_query(context, session, models.AccessInfo,
                                         marker, limit, sort_keys, sort_dirs,
                                         filters, offset,
                                         )
        if query is None:
            return []
        return query.all()


@apply_like_filters(model=models.AccessInfo)
def _process_access_info_filters(query, filters):
    """Common filter processing for AccessInfo queries."""
    if filters:
        if not is_valid_model_filters(models.AccessInfo, filters):
            return
        query = query.filter_by(**filters)

    return query


def storage_create(context, values):
    """Add a storage device from the values dictionary."""
    if not values.get('id'):
        values['id'] = uuidutils.generate_uuid()

    storage_ref = models.Storage()
    storage_ref.update(values)

    session = get_session()
    with session.begin():
        session.add(storage_ref)

    return _storage_get(context,
                        storage_ref['id'],
                        session=session)


def storage_update(context, storage_id, values):
    """Update a storage device with the values dictionary."""
    session = get_session()
    with session.begin():
        query = _storage_get_query(context, session)
        result = query.filter_by(id=storage_id).update(values)
    return result


def storage_get(context, storage_id):
    """Retrieve a storage device."""
    return _storage_get(context, storage_id)


def _storage_get(context, storage_id, session=None):
    result = (_storage_get_query(context, session=session)
              .filter_by(id=storage_id)
              .first())

    if not result:
        raise exception.StorageNotFound(storage_id)

    return result


def _storage_get_query(context, session=None):
    read_deleted = context.read_deleted
    kwargs = dict()

    if read_deleted in ('no', 'n', False):
        kwargs['deleted'] = False
    elif read_deleted in ('yes', 'y', True):
        kwargs['deleted'] = True

    return model_query(context, models.Storage, session=session, **kwargs)


def storage_get_all(context, marker=None, limit=None, sort_keys=None,
                    sort_dirs=None, filters=None, offset=None):
    session = get_session()
    with session.begin():
        # Generate the query
        query = _generate_paginate_query(context, session, models.Storage,
                                         marker, limit, sort_keys, sort_dirs,
                                         filters, offset,
                                         )
        # No storages   match, return empty list
        if query is None:
            return []
        return query.all()


@apply_like_filters(model=models.Storage)
def _process_storage_info_filters(query, filters):
    """Common filter processing for Storages queries."""
    if filters:
        if not is_valid_model_filters(models.Storage, filters):
            return
        query = query.filter_by(**filters)
    return query


def storage_delete(context, storage_id):
    """Delete a storage device."""
    delete_info = {'deleted': True, 'deleted_at': timeutils.utcnow()}
    _storage_get_query(context).filter_by(id=storage_id).update(delete_info)


def _volume_get_query(context, session=None):
    return model_query(context, models.Volume, session=session)


def _volume_get(context, volume_id, session=None):
    result = (_volume_get_query(context, session=session)
              .filter_by(id=volume_id)
              .first())

    if not result:
        raise exception.VolumeNotFound(volume_id)

    return result


def volume_create(context, values):
    """Create a volume."""
    if not values.get('id'):
        values['id'] = uuidutils.generate_uuid()

    vol_ref = models.Volume()
    vol_ref.update(values)

    session = get_session()
    with session.begin():
        session.add(vol_ref)

    return _volume_get(context,
                       vol_ref['id'],
                       session=session)


def volumes_create(context, volumes):
    """Create multiple volumes."""
    session = get_session()
    vol_refs = []
    with session.begin():

        for vol in volumes:
            LOG.debug('adding new volume for native_volume_id {0}:'
                      .format(vol.get('native_volume_id')))
            if not vol.get('id'):
                vol['id'] = uuidutils.generate_uuid()

            vol_ref = models.Volume()
            vol_ref.update(vol)
            vol_refs.append(vol_ref)

        session.add_all(vol_refs)

    return vol_refs


def volumes_delete(context, volumes_id_list):
    """Delete multiple volumes."""
    session = get_session()
    with session.begin():
        for vol_id in volumes_id_list:
            LOG.debug('deleting volume {0}:'.format(vol_id))
            query = _volume_get_query(context, session)
            result = query.filter_by(id=vol_id).delete()

            if not result:
                LOG.error(exception.VolumeNotFound(vol_id))
    return


def volume_update(context, vol_id, values):
    """Update a volume."""
    session = get_session()
    with session.begin():
        _volume_get(context, vol_id, session).update(values)
    return _volume_get(context, vol_id, session)


def volumes_update(context, volumes):
    """Update multiple volumes."""
    session = get_session()
    with session.begin():
        for vol in volumes:
            LOG.debug('updating volume {0}:'.format(vol.get('id')))
            query = _volume_get_query(context, session)
            result = query.filter_by(id=vol.get('id')
                                     ).update(vol)

            if not result:
                LOG.error(exception.VolumeNotFound(vol.get('id')))


def volume_get(context, volume_id):
    """Get a volume or raise an exception if it does not exist."""
    return _volume_get(context, volume_id)


def volume_get_all(context, marker=None, limit=None, sort_keys=None,
                   sort_dirs=None, filters=None, offset=None):
    """Retrieves all storage volumes."""
    session = get_session()
    with session.begin():
        # Generate the query
        query = _generate_paginate_query(context, session, models.Volume,
                                         marker, limit, sort_keys, sort_dirs,
                                         filters, offset)
        # No volume would match, return empty list
        if query is None:
            return []
        return query.all()


@apply_like_filters(model=models.Volume)
def _process_volume_info_filters(query, filters):
    """Common filter processing for volumes queries."""
    if filters:
        if not is_valid_model_filters(models.Volume, filters):
            return
        query = query.filter_by(**filters)

    return query


def volume_delete_by_storage(context, storage_id):
    """Delete all the volumes of a device"""
    _volume_get_query(context).filter_by(storage_id=storage_id).delete()


def _storage_pool_get_query(context, session=None):
    return model_query(context, models.StoragePool, session=session)


def _storage_pool_get(context, storage_pool_id, session=None):
    result = (_storage_pool_get_query(context, session=session)
              .filter_by(id=storage_pool_id)
              .first())

    if not result:
        raise exception.StoragePoolNotFound(storage_pool_id)

    return result


def storage_pool_create(context, values):
    """Create a storage_pool from the values dictionary."""
    if not values.get('id'):
        values['id'] = uuidutils.generate_uuid()

    storage_pool_ref = models.StoragePool()
    storage_pool_ref.update(values)

    session = get_session()
    with session.begin():
        session.add(storage_pool_ref)

    return _storage_pool_get(context,
                             storage_pool_ref['id'],
                             session=session)


def storage_pools_create(context, storage_pools):
    """Create a storage_pool from the values dictionary."""
    session = get_session()
    storage_pool_refs = []
    with session.begin():

        for storage_pool in storage_pools:
            LOG.debug('adding new storage_pool for native_storage_pool_id {0}:'
                      .format(storage_pool.get('native_storage_pool_id')))
            if not storage_pool.get('id'):
                storage_pool['id'] = uuidutils.generate_uuid()

            storage_pool_ref = models.StoragePool()
            storage_pool_ref.update(storage_pool)
            storage_pool_refs.append(storage_pool_ref)

        session.add_all(storage_pool_refs)

    return storage_pool_refs


def storage_pools_delete(context, storage_pools_id_list):
    """Delete multiple storage_pools with the storage_pools dictionary."""
    session = get_session()
    with session.begin():
        for storage_pool_id in storage_pools_id_list:
            LOG.debug('deleting storage_pool {0}:'.format(storage_pool_id))
            query = _storage_pool_get_query(context, session)
            result = query.filter_by(id=storage_pool_id).delete()

            if not result:
                LOG.error(exception.StoragePoolNotFound(storage_pool_id))

    return


def storage_pool_update(context, storage_pool_id, values):
    """Update a storage_pool withe the values dictionary."""
    session = get_session()

    with session.begin():
        query = _storage_pool_get_query(context, session)
        result = query.filter_by(id=storage_pool_id).update(values)

        if not result:
            raise exception.StoragePoolNotFound(storage_pool_id)

    return result


def storage_pools_update(context, storage_pools):
    """Update multiple storage_pools withe the storage_pools dictionary."""
    session = get_session()

    with session.begin():
        storage_pool_refs = []

        for storage_pool in storage_pools:
            LOG.debug('updating storage_pool {0}:'.format(
                storage_pool.get('id')))
            query = _storage_pool_get_query(context, session)
            result = query.filter_by(id=storage_pool.get('id')
                                     ).update(storage_pool)

            if not result:
                LOG.error(exception.StoragePoolNotFound(storage_pool.get(
                    'id')))
            else:
                storage_pool_refs.append(result)

    return storage_pool_refs


def storage_pool_get(context, storage_pool_id):
    """Get a storage_pool or raise an exception if it does not exist."""
    return _storage_pool_get(context, storage_pool_id)


def storage_pool_get_all(context, marker=None, limit=None, sort_keys=None,
                         sort_dirs=None, filters=None, offset=None):
    """Retrieves all storage storage_pools."""
    session = get_session()
    with session.begin():
        # Generate the query
        query = _generate_paginate_query(context, session, models.StoragePool,
                                         marker, limit, sort_keys, sort_dirs,
                                         filters, offset,
                                         )
        # No storage_pool would match, return empty list
        if query is None:
            return []
        return query.all()


def storage_pool_delete_by_storage(context, storage_id):
    """Delete all the storage_pools of a storage device"""
    _storage_pool_get_query(context).filter_by(storage_id=storage_id).delete()


@apply_like_filters(model=models.StoragePool)
def _process_storage_pool_info_filters(query, filters):
    """Common filter processing for storage_pools queries."""
    if filters:
        if not is_valid_model_filters(models.StoragePool, filters):
            return
        query = query.filter_by(**filters)

    return query


def disk_create(context, values):
    """Create a disk from the values dictionary."""
    return NotImplemented


def disk_update(context, disk_id, values):
    """Update a disk withe the values dictionary."""
    return NotImplemented


def disk_get(context, disk_id):
    """Get a disk or raise an exception if it does not exist."""
    return NotImplemented


def disk_get_all(context, marker=None, limit=None, sort_keys=None,
                 sort_dirs=None, filters=None, offset=None):
    """Retrieves all disks."""
    return NotImplemented


def is_orm_value(obj):
    """Check if object is an ORM field or expression."""
    return isinstance(obj, (sqlalchemy.orm.attributes.InstrumentedAttribute,
                            sqlalchemy.sql.expression.ColumnElement))


def model_query(context, model, *args, **kwargs):
    """Query helper for model query.

    :param context: context to query under
    :param model: model to query. Must be a subclass of ModelBase.
    :param session: if present, the session to use
    """
    session = kwargs.pop('session') or get_session()
    return db_utils.model_query(
        model=model, session=session, args=args, **kwargs)


def alert_source_get(context, storage_id):
    """Get an alert source or raise an exception if it does not exist."""
    return _alert_source_get(context, storage_id)


def _alert_source_get(context, storage_id, session=None):
    result = (_alert_source_get_query(context, session=session)
              .filter_by(storage_id=storage_id)
              .first())

    if not result:
        raise exception.AlertSourceNotFound(storage_id)

    return result


def _alert_source_get_query(context, session=None):
    return model_query(context, models.AlertSource, session=session)


@apply_like_filters(model=models.AlertSource)
def _process_alert_source_filters(query, filters):
    """Common filter processing for alert source queries."""
    if filters:
        if not is_valid_model_filters(models.AlertSource, filters):
            return
        query = query.filter_by(**filters)

    return query


def alert_source_create(context, values):
    """Add an alert source configuration."""
    alert_source_ref = models.AlertSource()
    alert_source_ref.update(values)

    session = get_session()
    with session.begin():
        session.add(alert_source_ref)

    return _alert_source_get(context,
                             alert_source_ref['storage_id'],
                             session=session)


def alert_source_update(context, storage_id, values):
    """Update an alert source configuration."""
    session = get_session()
    with session.begin():
        _alert_source_get(context, storage_id, session).update(values)
        return _alert_source_get(context, storage_id, session)


def alert_source_delete(context, storage_id):
    session = get_session()
    with session.begin():
        query = _alert_source_get_query(context, session)
        result = query.filter_by(storage_id=storage_id).delete()
        if not result:
            LOG.error("Cannot delete non-exist alert source[storage_id=%s]." %
                      storage_id)
            raise exception.AlertSourceNotFound(storage_id)
        else:
            LOG.info("Delete alert source[storage_id=%s] successfully." %
                     storage_id)


def alert_source_get_all(context, marker=None, limit=None, sort_keys=None,
                         sort_dirs=None, filters=None, offset=None):
    session = get_session()
    with session.begin():
        query = _generate_paginate_query(context, session, models.AlertSource,
                                         marker, limit, sort_keys, sort_dirs,
                                         filters, offset)
        if query is None:
            return []
        return query.all()


PAGINATION_HELPERS = {
    models.AccessInfo: (_access_info_get_query, _process_access_info_filters,
                        _access_info_get),
    models.StoragePool: (_storage_pool_get_query,
                         _process_storage_pool_info_filters,
                         _storage_pool_get),
    models.Storage: (_storage_get_query, _process_storage_info_filters,
                     _storage_get),
    models.AlertSource: (_alert_source_get_query,
                         _process_alert_source_filters,
                         _alert_source_get),
    models.Volume: (_volume_get_query, _process_volume_info_filters,
                    _volume_get),
}


def process_sort_params(sort_keys, sort_dirs, default_keys=None,
                        default_dir='asc'):
    """Process the sort parameters to include default keys.

    Creates a list of sort keys and a list of sort directions. Adds the default
    keys to the end of the list if they are not already included.

    When adding the default keys to the sort keys list, the associated
    direction is:
    1) The first element in the 'sort_dirs' list (if specified), else
    2) 'default_dir' value (Note that 'asc' is the default value since this is
    the default in sqlalchemy.utils.paginate_query)

    :param sort_keys: List of sort keys to include in the processed list
    :param sort_dirs: List of sort directions to include in the processed list
    :param default_keys: List of sort keys that need to be included in the
                         processed list, they are added at the end of the list
                         if not already specified.
    :param default_dir: Sort direction associated with each of the default
                        keys that are not supplied, used when they are added
                        to the processed list
    :returns: list of sort keys, list of sort directions
    :raise exception.InvalidInput: If more sort directions than sort keys
                                   are specified or if an invalid sort
                                   direction is specified
    """
    if default_keys is None:
        default_keys = ['created_at']

    # Determine direction to use for when adding default keys
    if sort_dirs and len(sort_dirs):
        default_dir_value = sort_dirs[0]
    else:
        default_dir_value = default_dir

    # Create list of keys (do not modify the input list)
    if sort_keys:
        result_keys = list(sort_keys)
    else:
        result_keys = []

    # If a list of directions is not provided, use the default sort direction
    # for all provided keys.
    if sort_dirs:
        result_dirs = []
        # Verify sort direction
        for sort_dir in sort_dirs:
            if sort_dir not in ('asc', 'desc'):
                msg = _("Unknown sort direction, must be 'desc' or 'asc'.")
                raise exception.InvalidInput(msg)
            result_dirs.append(sort_dir)
    else:
        result_dirs = [default_dir_value for _sort_key in result_keys]

    # Ensure that the key and direction length match
    while len(result_dirs) < len(result_keys):
        result_dirs.append(default_dir_value)
    # Unless more direction are specified, which is an error
    if len(result_dirs) > len(result_keys):
        msg = _("Sort direction array size exceeds sort key array size.")
        raise exception.InvalidInput(msg)

    # Ensure defaults are included
    for key in default_keys:
        if key not in result_keys:
            result_keys.append(key)
            result_dirs.append(default_dir_value)

    return result_keys, result_dirs


def _generate_paginate_query(context, session, paginate_type, marker,
                             limit, sort_keys, sort_dirs, filters,
                             offset=None
                             ):
    """Generate the query to include the filters and the paginate options.

    Returns a query with sorting / pagination criteria added or None
    if the given filters will not yield any results.

    :param context: context to query under
    :param session: the session to use
    :param marker: the last item of the previous page; we returns the next
                    results after this value.
    :param limit: maximum number of items to return
    :param sort_keys: list of attributes by which results should be sorted,
                      paired with corresponding item in sort_dirs
    :param sort_dirs: list of directions in which results should be sorted,
                      paired with corresponding item in sort_keys
    :param filters: dictionary of filters; values that are in lists, tuples,
                    or sets cause an 'IN' operation, while exact matching
                    is used for other values, see _process_volume_filters
                    function for more information
    :param offset: number of items to skip
    :param paginate_type: type of pagination to generate
    :returns: updated query or None
    """
    get_query, process_filters, get = PAGINATION_HELPERS[paginate_type]

    sort_keys, sort_dirs = process_sort_params(sort_keys,
                                               sort_dirs,
                                               default_dir='desc')
    query = get_query(context, session=session)

    if filters:
        query = process_filters(query, filters)
        if query is None:
            return None

    marker_object = None
    if marker is not None:
        marker_object = get(context, marker, session)

    return sqlalchemyutils.paginate_query(query, paginate_type, limit,
                                          sort_keys,
                                          marker=marker_object,
                                          sort_dirs=sort_dirs,
                                          offset=offset)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import six

from oslo_log import log
from oslo_utils import uuidutils

from delfin.drivers import helper
from delfin.drivers import manager

LOG = log.getLogger(__name__)


class API(object):
    def __init__(self):
        self.driver_manager = manager.DriverManager()

    def discover_storage(self, context, access_info):
        """Discover a storage system with access information."""
        if 'storage_id' not in access_info:
            access_info['storage_id'] = six.text_type(
                uuidutils.generate_uuid())

        driver = self.driver_manager.get_driver(context,
                                                cache_on_load=False,
                                                **access_info)
        storage = driver.get_storage(context)

        # Need to validate storage response from driver
        helper.check_storage_repetition(context, storage)
        access_info = helper.create_access_info(context, access_info)
        storage['id'] = access_info['storage_id']
        storage = helper.create_storage(context, storage)
        self.driver_manager.update_driver(storage['id'], driver)

        LOG.info("Storage found successfully.")
        return storage

    def update_access_info(self, context, access_info):
        """Validate and update access information."""
        driver = self.driver_manager.get_driver(context,
                                                cache_on_load=False,
                                                **access_info)
        storage_new = driver.get_storage(context)

        # Need to validate storage response from driver
        storage_id = access_info['storage_id']
        helper.check_storage_consistency(context, storage_id, storage_new)
        access_info = helper.update_access_info(context,
                                                storage_id, access_info)
        helper.update_storage(context, storage_id, storage_new)
        self.driver_manager.update_driver(storage_id, driver)

        LOG.info("Access information updated successfully.")
        return access_info

    def remove_storage(self, context, storage_id):
        """Clear driver instance from driver factory."""
        self.driver_manager.remove_driver(storage_id)

    def get_storage(self, context, storage_id):
        """Get storage device information from storage system"""
        driver = self.driver_manager.get_driver(context, storage_id=storage_id)
        return driver.get_storage(context)

    def list_storage_pools(self, context, storage_id):
        """List all storage pools from storage system."""
        driver = self.driver_manager.get_driver(context, storage_id=storage_id)
        return driver.list_storage_pools(context)

    def list_volumes(self, context, storage_id):
        """List all storage volumes from storage system."""
        driver = self.driver_manager.get_driver(context, storage_id=storage_id)
        return driver.list_volumes(context)

    def add_trap_config(self, context, storage_id, trap_config):
        """Config the trap receiver in storage system."""
        pass

    def remove_trap_config(self, context, storage_id, trap_config):
        """Remove trap receiver configuration from storage system."""
        pass

    def parse_alert(self, context, storage_id, alert):
        """Parse alert data got from snmp trap server."""
        driver = self.driver_manager.get_driver(context, storage_id=storage_id)
        return driver.parse_alert(context, alert)

    def clear_alert(self, context, storage_id, sequence_number):
        """Clear alert from storage system."""
        driver = self.driver_manager.get_driver(context, storage_id=storage_id)
        return driver.clear_alert(context, sequence_number)

    def list_alerts(self, context, storage_id):
        """List alert from storage system."""
        driver = self.driver_manager.get_driver(context, storage_id=storage_id)
        return driver.list_alerts(context)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import copy
import six
import stevedore
import threading

from oslo_log import log

from delfin import exception
from delfin import utils
from delfin import ssl_utils
from delfin.drivers import helper

LOG = log.getLogger(__name__)


@six.add_metaclass(utils.Singleton)
class DriverManager(stevedore.ExtensionManager):
    _instance_lock = threading.Lock()
    NAMESPACE = 'delfin.storage.drivers'

    def __init__(self):
        super(DriverManager, self).__init__(self.NAMESPACE)
        # The driver_factory will keep the driver instance for
        # each of storage systems so that the session between driver
        # and storage system is effectively used.
        self.driver_factory = dict()

    def get_driver(self, context, invoke_on_load=True,
                   cache_on_load=True, **kwargs):
        """Get a driver from manager.

        :param context: The context of delfin.
        :type context: delfin.context.RequestContext
        :param invoke_on_load: Boolean to decide whether to return the
            driver object.
        :type invoke_on_load: bool
        :param cache_on_load: Boolean to decide whether save driver object
            in driver_factory when generating a new driver object.
            It takes effect when invoke_on_load is True.
        :type cache_on_load: bool
        :param kwargs: Parameters from access_info.
        """
        kwargs['verify'] = False
        ca_path = ssl_utils.get_storage_ca_path()
        if ca_path:
            ssl_utils.verify_ca_path(ca_path)
            kwargs['verify'] = ca_path

        if not invoke_on_load:
            return self._get_driver_cls(**kwargs)
        else:
            return self._get_driver_obj(context, cache_on_load, **kwargs)

    def update_driver(self, storage_id, driver):
        self.driver_factory[storage_id] = driver

    def remove_driver(self, storage_id):
        """Clear driver instance from driver factory."""
        self.driver_factory.pop(storage_id, None)

    def _get_driver_obj(self, context, cache_on_load=True, **kwargs):
        if not cache_on_load or not kwargs.get('storage_id'):
            if kwargs['verify']:
                ssl_utils.reload_certificate(kwargs['verify'])
            cls = self._get_driver_cls(**kwargs)
            return cls(**kwargs)

        with self._instance_lock:
            if kwargs['storage_id'] in self.driver_factory:
                driver = self.driver_factory[kwargs['storage_id']]
                return driver

            if kwargs['verify']:
                ssl_utils.reload_certificate(kwargs['verify'])
            access_info = copy.deepcopy(kwargs)
            storage_id = access_info.pop('storage_id')
            access_info.pop('verify')
            if access_info:
                cls = self._get_driver_cls(**kwargs)
                driver = cls(**kwargs)
            else:
                access_info = helper.get_access_info(context, storage_id)
                access_info['verify'] = kwargs.get('verify')
                cls = self._get_driver_cls(**access_info)
                driver = cls(**access_info)

            self.driver_factory[storage_id] = driver
            return driver

    def _get_driver_cls(self, **kwargs):
        """Get driver class from entry points."""
        name = '%s %s' % (kwargs.get('vendor'), kwargs.get('model'))
        if name in self.names():
            return self[name].plugin

        msg = "Storage driver '%s' could not be found." % name
        LOG.error(msg)
        raise exception.StorageDriverNotFound(name)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect

import decorator
from oslo_log import log

from delfin import coordination
from delfin import db
from delfin import exception
from delfin.common import constants
from delfin.drivers import api as driverapi
from delfin.i18n import _

LOG = log.getLogger(__name__)


def set_synced_after():
    @decorator.decorator
    def _set_synced_after(func, *args, **kwargs):
        call_args = inspect.getcallargs(func, *args, **kwargs)
        self = call_args['self']
        ret = func(*args, **kwargs)
        lock = coordination.Lock(self.storage_id)
        with lock:
            try:
                storage = db.storage_get(self.context, self.storage_id)
            except exception.StorageNotFound:
                LOG.warn('Storage %s not found when set synced'
                         % self.storage_id)
            else:
                # One sync task done, sync status minus 1
                # When sync status get to 0
                # means all the sync tasks are completed
                if storage['sync_status'] != constants.SyncStatus.SYNCED:
                    storage['sync_status'] -= 1
                    db.storage_update(self.context, self.storage_id, storage)

        return ret

    return _set_synced_after


def check_deleted():
    @decorator.decorator
    def _check_deleted(func, *args, **kwargs):
        call_args = inspect.getcallargs(func, *args, **kwargs)
        self = call_args['self']
        ret = func(*args, **kwargs)
        # When context.read_deleted = 'yes', db.storage_get would
        # only get the storage whose 'deleted' tag is not default value
        self.context.read_deleted = 'yes'
        try:
            db.storage_get(self.context, self.storage_id)
        except exception.StorageNotFound:
            LOG.debug('Storage %s not found when checking deleted'
                      % self.storage_id)
        else:
            self.remove()
        self.context.read_deleted = 'no'
        return ret

    return _check_deleted


class StorageResourceTask(object):

    def __init__(self, context, storage_id):
        self.storage_id = storage_id
        self.context = context
        self.driver_api = driverapi.API()

    def _classify_resources(self, storage_resources, db_resources, key):
        """
        :param storage_resources:
        :param db_resources:
        :return: it will return three list add_list: the items present in
        storage but not in current_db. update_list:the items present in
        storage and in current_db. delete_id_list:the items present not in
        storage but present in current_db.
        """
        original_ids_in_db = [resource[key]
                              for resource in db_resources]
        delete_id_list = [resource['id'] for resource in db_resources]
        add_list = []
        update_list = []

        for resource in storage_resources:
            if resource[key] in original_ids_in_db:
                resource['id'] = db_resources[original_ids_in_db.index(
                    resource[key])]['id']
                delete_id_list.remove(resource['id'])
                update_list.append(resource)
            else:
                add_list.append(resource)

        return add_list, update_list, delete_id_list


class StorageDeviceTask(StorageResourceTask):
    def __init__(self, context, storage_id):
        super(StorageDeviceTask, self).__init__(context, storage_id)

    @check_deleted()
    @set_synced_after()
    def sync(self):
        """
        :return:
        """
        LOG.info('Syncing storage device for storage id:{0}'.format(
            self.storage_id))
        try:
            storage = self.driver_api.get_storage(self.context,
                                                  self.storage_id)

            db.storage_update(self.context, self.storage_id, storage)
        except AttributeError as e:
            LOG.error(e)
        except Exception as e:
            msg = _('Failed to update storage entry in DB: {0}'
                    .format(e))
            LOG.error(msg)
        else:
            LOG.info("Syncing storage successful!!!")

    def remove(self):
        LOG.info('Remove storage device for storage id:{0}'
                 .format(self.storage_id))
        try:
            db.storage_delete(self.context, self.storage_id)
            db.access_info_delete(self.context, self.storage_id)
            db.alert_source_delete(self.context, self.storage_id)
        except Exception as e:
            LOG.error('Failed to update storage entry in DB: {0}'.format(e))


class StoragePoolTask(StorageResourceTask):
    def __init__(self, context, storage_id):
        super(StoragePoolTask, self).__init__(context, storage_id)

    @check_deleted()
    @set_synced_after()
    def sync(self):
        """
        :return:
        """
        LOG.info('Syncing storage pool for storage id:{0}'.format(
            self.storage_id))
        try:
            # collect the storage pools list from driver and database
            storage_pools = self.driver_api.list_storage_pools(self.context,
                                                               self.storage_id)
            db_pools = db.storage_pool_get_all(self.context,
                                               filters={"storage_id":
                                                        self.storage_id})

            add_list, update_list, delete_id_list = self._classify_resources(
                storage_pools, db_pools, 'native_storage_pool_id'
            )
            if delete_id_list:
                db.storage_pools_delete(self.context, delete_id_list)

            if update_list:
                db.storage_pools_update(self.context, update_list)

            if add_list:
                db.storage_pools_create(self.context, add_list)
        except AttributeError as e:
            LOG.error(e)
        except Exception as e:
            msg = _('Failed to sync pools entry in DB: {0}'
                    .format(e))
            LOG.error(msg)
        else:
            LOG.info("Syncing storage pools successful!!!")

    def remove(self):
        LOG.info('Remove storage pools for storage id:{0}'.format(
            self.storage_id))
        db.storage_pool_delete_by_storage(self.context, self.storage_id)


class StorageVolumeTask(StorageResourceTask):
    def __init__(self, context, storage_id):
        super(StorageVolumeTask, self).__init__(context, storage_id)

    @check_deleted()
    @set_synced_after()
    def sync(self):
        """
        :return:
        """
        LOG.info('Syncing volumes for storage id:{0}'.format(self.storage_id))
        try:
            # collect the volumes list from driver and database
            storage_volumes = self.driver_api.list_volumes(self.context,
                                                           self.storage_id)
            db_volumes = db.volume_get_all(self.context,
                                           filters={"storage_id":
                                                    self.storage_id})

            add_list, update_list, delete_id_list = self._classify_resources(
                storage_volumes, db_volumes, 'native_volume_id'
            )
            LOG.info('###StorageVolumeTask for {0}:add={1},delete={2},'
                     'update={3}'.format(self.storage_id,
                                         len(add_list),
                                         len(delete_id_list),
                                         len(update_list)))
            if delete_id_list:
                db.volumes_delete(self.context, delete_id_list)

            if update_list:
                db.volumes_update(self.context, update_list)

            if add_list:
                db.volumes_create(self.context, add_list)
        except AttributeError as e:
            LOG.error(e)
        except Exception as e:
            msg = _('Failed to sync volumes entry in DB: {0}'
                    .format(e))
            LOG.error(msg)
        else:
            LOG.info("Syncing volumes successful!!!")

    def remove(self):
        LOG.info('Remove volumes for storage id:{0}'.format(self.storage_id))
        db.volume_delete_by_storage(self.context, self.storage_id)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin.common import constants


class ViewBuilder(object):

    def show(self, access_info):
        access_info_dict = access_info.to_dict()
        for access in constants.ACCESS_TYPE:
            if access_info.get(access):
                access_info[access].pop('password', None)
        return access_info_dict



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from unittest import mock

from oslo_utils import importutils

from delfin import context
from delfin import exception
from delfin.tests.unit.api import fakes


class AlertControllerTestCase(unittest.TestCase):
    ALERT_CONTROLLER_CLASS = 'delfin.api.v1.alerts.AlertController'

    @mock.patch('delfin.alert_manager.rpcapi.AlertAPI', mock.Mock())
    def _get_alert_controller(self):
        alert_controller_class = importutils.import_class(
            self.ALERT_CONTROLLER_CLASS)
        alert_controller = alert_controller_class()
        return alert_controller

    @mock.patch('delfin.db.storage_get', fakes.fake_storages_get_all)
    @mock.patch('delfin.drivers.api.API.clear_alert')
    def test_delete_alert_success(self, mock_clear_alert):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alerts'
                                      '/fake_sequence_number')
        fake_storage_id = 'abcd-1234-5678'
        fake_sequence_number = 'abcd-1234'

        alert_controller_inst = self._get_alert_controller()
        alert_controller_inst.delete(req, fake_storage_id,
                                     fake_sequence_number)
        self.assertTrue(mock_clear_alert.called_with(context, fake_storage_id,
                                                     fake_sequence_number))

    @mock.patch('delfin.db.storage_get', fakes.fake_storage_get_exception)
    @mock.patch('delfin.drivers.api.API.clear_alert', mock.Mock())
    def test_delete_alert_failure_storage_not_found(self):
        req = fakes.HTTPRequest.blank('/storages/fake_id/alerts'
                                      '/fake_sequence_number')
        fake_storage_id = 'abcd-1234-5678'
        fake_sequence_number = 'abcd-1234'

        alert_controller_inst = self._get_alert_controller()
        self.assertRaisesRegex(exception.StorageNotFound, "Storage "
                                                          "abcd-1234-5678 "
                                                          "could not be "
                                                          "found",
                               alert_controller_inst.delete, req,
                               fake_storage_id, fake_sequence_number)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from oslo_config import cfg
from oslo_log import log
import six
from stevedore import extension

from delfin import exception
from delfin.i18n import _

LOG = log.getLogger(__name__)

exporter_opts = [
    cfg.ListOpt('alert_exporters',
                default=['AlertExporterExample'],
                help="Which exporters for alert push."),
    cfg.ListOpt('performance_exporters',
                default=['PerformanceExporterExample'],
                help="Which exporters for performance push."),
]

CONF = cfg.CONF
CONF.register_opts(exporter_opts)


class BaseExporter(object):
    """Base class for data exporter."""

    def dispatch(self, ctxt, data):
        """Dispatch data to the third platforms.
            :param ctxt: delfin.RequestContext
            :param data: The data to be pushed, it's a list with dict item.
            :type data: list
        """
        raise NotImplementedError()


class BaseManager(BaseExporter):
    def __init__(self, namespace):
        self.extension_manager = extension.ExtensionManager(namespace)
        self.exporters = self._get_exporters()

    def dispatch(self, ctxt, data):
        if not isinstance(data, (list, tuple)):
            data = [data]
        for exporter in self.exporters:
            try:
                exporter.dispatch(ctxt, data)
            except exception.DelfinException as e:
                err_msg = _("Failed to export data (%s).") % e.msg
                LOG.exception(err_msg)
            except Exception as e:
                err_msg = six.text_type(e)
                LOG.exception(err_msg)

    def _get_exporters(self):
        """Get exporters from configuration file which
        shall be supported in entry points.
        """
        supported_exporters = self._get_supported_exporters()
        configured_exporters = self._get_configured_exporters()
        return [cls() for cls in supported_exporters
                if cls.__name__ in configured_exporters]

    def _get_supported_exporters(self):
        """Get all supported exporters from entry points file."""
        return [ext.plugin for ext in self.extension_manager]

    def _get_configured_exporters(self):
        """Get exporters from configuration file."""
        raise NotImplementedError()


class AlertExporterManager(BaseManager):
    NAMESPACE = 'delfin.alert.exporters'

    def __init__(self):
        super(AlertExporterManager, self).__init__(self.NAMESPACE)

    def _get_configured_exporters(self):
        return CONF.alert_exporters


class PerformanceExporterManager(BaseManager):
    NAMESPACE = 'delfin.performance.exporters'

    def __init__(self):
        super(PerformanceExporterManager, self).__init__(self.NAMESPACE)

    def _get_configured_exporters(self):
        return CONF.performance_exporters



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Client side of the task manager RPC API.
"""

import oslo_messaging as messaging
from oslo_config import cfg

from delfin import rpc

CONF = cfg.CONF


class TaskAPI(object):
    """Client side of the task rpc API.

    API version history:

        1.0 - Initial version.
    """

    RPC_API_VERSION = '1.0'

    def __init__(self):
        super(TaskAPI, self).__init__()
        target = messaging.Target(topic=CONF.delfin_task_topic,
                                  version=self.RPC_API_VERSION)
        self.client = rpc.get_client(target, version_cap=self.RPC_API_VERSION)

    def sync_storage_resource(self, context, storage_id, resource_task):
        call_context = self.client.prepare(version='1.0')
        return call_context.cast(context,
                                 'sync_storage_resource',
                                 storage_id=storage_id,
                                 resource_task=resource_task)

    def remove_storage_resource(self, context, storage_id, resource_task):
        call_context = self.client.prepare(version='1.0')
        return call_context.cast(context,
                                 'remove_storage_resource',
                                 storage_id=storage_id,
                                 resource_task=resource_task)

    def remove_storage_in_cache(self, context, storage_id):
        call_context = self.client.prepare(version='1.0', fanout=True)
        return call_context.cast(context,
                                 'remove_storage_in_cache',
                                 storage_id=storage_id)
